@@ -5,7 +5,7 @@
  **   Providing a flexible user environment				     **
  ** 									     **
  **   File:		cmdModule.c					     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Authors:	John Furlan, jlf@behere.com				     **
  **		Jens Hamisch, jens@Strawberry.COM			     **
@@ -31,7 +31,7 @@
  ** 									     ** 
  ** ************************************************************************ **/
 
-static char Id[] = "@(#)$Id: cmdModule.c,v 1.4 2001/07/11 20:11:02 rkowen Exp $";
+static char Id[] = "@(#)$Id: cmdModule.c,v 1.11.2.2 2006/02/21 22:26:56 rkowen Exp $";
 static void *UseId[] = { &UseId, Id };
 
 /** ************************************************************************ **/
@@ -99,7 +99,7 @@ char	 *module_command;
  **   Description:	Evaluation of the module command line and callup of  **
  **			the according subroutine			     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	ClientData	 client_data			     **
  **			Tcl_Interp	*interp		According Tcl interp.**
@@ -121,16 +121,12 @@ char	 *module_command;
 int	cmdModule(	ClientData	 client_data,
 	       		Tcl_Interp	*interp,
 	       		int		 argc,
-	       		char		*argv[])
+	       		CONST84 char	*argv[])
 {
     int		  return_val = -1, i;
     int		  store_flags = g_flags;
     char	 *store_curmodule = NULL;
     char	 *save_module_command = NULL;
-#ifdef FORCE_PATH
-    char	 *base_path = NULL;
-    char	 *sacred_path = NULL;
-#endif
     int 	  match = 0;
 
     /**
@@ -138,7 +134,7 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     int		  num_modulefiles = argc - 2;
-    char	**modulefile_list = argv + 2;
+    char	**modulefile_list = (char **) argv + 2;
 
 #if 0
 	int x=0;
@@ -148,7 +144,8 @@ int	cmdModule(	ClientData	 client_data,
 #endif
 
 #define	_MTCH	_XD match =
-#define	_ISERR	((match == -1) && (*interp->result))
+#define	_ISERR	((match == -1) && (*TCL_RESULT(interp)))
+#define _TCLCHK(a) {if(_ISERR) ErrorLogger(ERR_EXEC,LOC,TCL_RESULT(a),NULL);}
 
 #if WITH_DEBUGGING_CALLBACK
     ErrorLogger( NO_ERR_START, LOC, _proc_cmdModule, NULL);
@@ -166,12 +163,20 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     if( argc < 2) {
-	(void) ErrorLogger( ERR_USAGE, LOC, argv[0], " subcommand ",
+	(void) ErrorLogger( ERR_USAGE, LOC, "module", " command ",
 	    " [arguments ...] ", NULL);
 	(void) ModuleCmd_Help( interp, 0, modulefile_list);
 	return( TCL_ERROR);		/** -------- EXIT (FAILURE) -------> **/
     }
 
+    /**
+     **  Non-persist mode?
+     **/
+    
+    if (g_flags & M_NONPERSIST) {
+	return (TCL_OK);
+    }
+
     /**
      **  Display whatis mode?
      **/
@@ -208,42 +213,9 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command, addRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	return_val = ModuleCmd_Load( interp, 1,num_modulefiles,modulefile_list);
 
-#ifdef FORCE_PATH
-        if( return_val) {
-
-           /**
-            **  return_val now indicates whether ANY modulefile was loaded
-            **  Only if any modulefile was loaded do we need to worry
-            **  about forcing paths.
-            **/
-
-            if( base_path = (char *) getenv( "MODULES_PATH_BASE") ||
-                base_path = (char *) getenv( "BASE_PATH")) {
-                ForceBasePath( interp, "PATH", base_path);
-            } else {
-                ForceBasePath( interp, "PATH", FORCE_PATH);
-            }
-
-            if( sacred_path = (char *) getenv( "MODULES_PATH_SACRED") ||
-                sacred_path = (char *) getenv( "SACRED_PATH")) {
-                ForceSacredPath( interp, "PATH", sacred_path);
-            } else {
-                ForceSacredPath( interp, "PATH", FORCE_PATH_SACRED);
-            }
-
-            if(sacred_path = (char *) getenv("MODULES_LD_LIBRARY_PATH_SACRED")){
-                ForceSacredPath( interp, "LD_LIBRARY_PATH", sacred_path);
-            } else {
-                ForceSacredPath( interp, "LD_LIBRARY_PATH", 
-		    FORCE_LD_LIBRARY_PATH_SACRED);
-            }
-
-        } /** if( return_val) **/
-#endif
-
        /**
         **  We always say the load succeeded.  ModuleCmd_Load will
         **  output any necessary error messages.
@@ -256,7 +228,7 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command, rmRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
         ModuleCmd_Load( interp, 0, num_modulefiles, modulefile_list);
 	return_val = TCL_OK;
 
@@ -265,7 +237,7 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command, swRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	return_val = ModuleCmd_Switch( interp, num_modulefiles,modulefile_list);
 
     /**
@@ -273,7 +245,7 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command, dispRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	return_val = ModuleCmd_Display( interp,num_modulefiles,modulefile_list);
 
     /**
@@ -281,7 +253,7 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command, listRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	if (! (sw_format & SW_SET) ) {	/* default format options */
 		sw_format |= (SW_HUMAN | SW_TERSE );
 		sw_format &= ~(SW_PARSE | SW_LONG );
@@ -295,7 +267,7 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command,availRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	if (! (sw_format & SW_SET) ) {	/* default format options */
 		sw_format |= (SW_HUMAN | SW_TERSE);
 		sw_format &= ~(SW_PARSE | SW_LONG );
@@ -309,11 +281,11 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command,whatisRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	return_val = ModuleCmd_Whatis(interp, num_modulefiles, modulefile_list);
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command,aproposRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	return_val = ModuleCmd_Apropos(interp, num_modulefiles,modulefile_list);
 
     /**
@@ -321,7 +293,7 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command,clearRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	return_val = ModuleCmd_Clear( interp, num_modulefiles, modulefile_list);
 
     /**
@@ -329,7 +301,7 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command,updateRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	return_val = ModuleCmd_Update(interp, num_modulefiles, modulefile_list);
 
     /**
@@ -337,7 +309,7 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command,purgeRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	return_val = ModuleCmd_Purge( interp, num_modulefiles, modulefile_list);
 
     /**
@@ -345,45 +317,45 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command,initRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	
         if( Tcl_RegExpMatch(interp,module_command, "^inita|^ia")){/* initadd */
-	    if (_ISERR) ErrorLogger(ERR_EXEC,LOC,interp->result,NULL);
+	    _TCLCHK(interp);
 	    g_flags |= M_LOAD;
 	    return_val = ModuleCmd_Init(interp,num_modulefiles,modulefile_list);
 	    g_flags &= ~M_LOAD;
 	}
 	
         if( Tcl_RegExpMatch(interp,module_command, "^initr|^iw")){ /* initrm */
-	    if (_ISERR) ErrorLogger(ERR_EXEC,LOC,interp->result,NULL);
+	    _TCLCHK(interp);
 	    g_flags |= M_REMOVE;
 	    return_val = ModuleCmd_Init(interp,num_modulefiles,modulefile_list);
 	    g_flags &= ~M_REMOVE;
 	}
 	
         if( Tcl_RegExpMatch(interp,module_command, "^initl|^il")){/* initlist*/
-	    if (_ISERR) ErrorLogger(ERR_EXEC,LOC,interp->result,NULL);
+	    _TCLCHK(interp);
 	    g_flags |= M_DISPLAY;
 	    return_val = ModuleCmd_Init(interp,num_modulefiles,modulefile_list);
 	    g_flags &= ~M_DISPLAY;
 	}
 	
         if(Tcl_RegExpMatch(interp,module_command, "^inits|^is")){/* initswitch*/
-	    if (_ISERR) ErrorLogger(ERR_EXEC,LOC,interp->result,NULL);
+	    _TCLCHK(interp);
 	    g_flags |= M_SWITCH;
 	    return_val = ModuleCmd_Init(interp,num_modulefiles,modulefile_list);
 	    g_flags &= ~M_SWITCH;
 	}
 	
         if(Tcl_RegExpMatch(interp,module_command, "^initc|^ic")){/* initclear*/
-	    if (_ISERR) ErrorLogger(ERR_EXEC,LOC,interp->result,NULL);
+	    _TCLCHK(interp);
 	    g_flags |= M_CLEAR;
 	    return_val = ModuleCmd_Init(interp,num_modulefiles,modulefile_list);
 	    g_flags &= ~M_CLEAR;
 	}
 	
         if(Tcl_RegExpMatch(interp,module_command,"^initp|^ip")){/*initprepend*/
-	    if (_ISERR) ErrorLogger(ERR_EXEC,LOC,interp->result,NULL);
+	    _TCLCHK(interp);
 	    g_flags |= (M_PREPEND | M_LOAD);
 	    return_val = ModuleCmd_Init(interp,num_modulefiles,modulefile_list);
 	    g_flags &= ~(M_PREPEND | M_LOAD);
@@ -394,7 +366,7 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command, useRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	return_val = ModuleCmd_Use( interp, num_modulefiles, modulefile_list);
 
     /**
@@ -402,15 +374,23 @@ int	cmdModule(	ClientData	 client_data,
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command, unuseRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	return_val = ModuleCmd_UnUse( interp, num_modulefiles, modulefile_list);
 
+    /**
+     **  --- module REFRESH 
+     **/
+
+    } else if(_MTCH Tcl_RegExpMatch(interp,module_command, refreshRE)) {
+	_TCLCHK(interp);
+	return_val = ModuleCmd_Refresh( interp, num_modulefiles, modulefile_list);
+
     /**
      **  --- module HELP
      **/
 
     } else if(_MTCH Tcl_RegExpMatch(interp,module_command, helpRE)) {
-	if (_ISERR) ErrorLogger( ERR_EXEC, LOC, interp->result, NULL);
+	_TCLCHK(interp);
 	return_val = ModuleCmd_Help( interp, num_modulefiles, modulefile_list);
     }
     
@@ -420,7 +400,7 @@ int	cmdModule(	ClientData	 client_data,
 
     if( CheckTracingList(interp,  module_command,
 	num_modulefiles, modulefile_list))
-	Module_Tracing( return_val, argc, argv);
+	Module_Tracing( return_val, argc, (char **) argv);
 
     /**
      **  Evaluate the subcommands return value in order to get rid of unrecog-
@@ -461,12 +441,13 @@ int	cmdModule(	ClientData	 client_data,
  **   Description:	Check the passed filename for to be a valid module   **
  **			and execute the according command file		     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		According Tcl interp.**
  **		 	char		*filename			     **
  ** 									     **
  **   Result:		int	TCL_OK		Successfull completion	     **
+ **				TCL_BREAK	break command		     **
  **				TCL_ERROR	Any error		     **
  ** 									     **
  **   Attached Globals:							     **
@@ -509,32 +490,17 @@ int   Read_Modulefile( Tcl_Interp	*interp,
      **  latest executed command
      **/
 
-    if( TCL_ERROR == (result = Execute_TclFile(interp, filename))) {
+    result = Execute_TclFile(interp, filename);
 
 #if WITH_DEBUGGING_UTIL
+    if(EM_ERROR == ReturnValue(interp, result))
 	ErrorLogger( NO_ERR_DEBUG, LOC, "Execution of '",
 		filename, "' failed", NULL);
 #endif
 
-	if( *interp->result) {
-	    char *tstr = NULL;
-	    Tcl_RegExp retexpPtr;
-
-	    tstr = strdup(interp->result);
-	    retexpPtr = Tcl_RegExpCompile(interp, "^EXIT ([0-9]*)");
-	    if( Tcl_RegExpExec(interp, retexpPtr, tstr, tstr)) {
-		Tcl_RegExpRange(retexpPtr, 1, &startp, &endp);
-		if( startp != '\0')
-		    result = atoi( startp );
-	    }
-	    if (tstr) free(tstr);
-	}
-    }
-
     /**
      **  Return the result as derivered from the module file execution
      **/
-
 #if WITH_DEBUGGING_UTIL
     ErrorLogger( NO_ERR_END, LOC, _proc_Read_Modulefile, NULL);
 #endif
@@ -551,7 +517,7 @@ int   Read_Modulefile( Tcl_Interp	*interp,
  **   Description:	Read in and execute all commands concerning the Tcl  **
  **			file passed as parameter			     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		According Tcl interp.**
  **		 	char		*filename			     **
@@ -570,6 +536,7 @@ int	 Execute_TclFile(	Tcl_Interp	*interp,
     FILE	*infile;
     int		 gotPartial = 0;
     int		 result = 0;
+    EM_RetVal	 em_result = EM_OK;
     char	*cmd;
     Tcl_DString	 cmdbuf;
 
@@ -642,7 +609,7 @@ int	 Execute_TclFile(	Tcl_Interp	*interp,
 
         result = Tcl_Eval( interp, cmd);
 
-	if( TCL_ERROR == result) {
+	if( EM_ERROR == (em_result = ReturnValue(interp, result))) {
 	    ErrorLogger( ERR_EXEC, LOC, cmd, NULL);
 	}
 
@@ -717,7 +684,7 @@ int	 Execute_TclFile(	Tcl_Interp	*interp,
  **			Executes the passed modulefile (conditionally hidden)**
  **			and then evaluates the passed Tcl procedure	     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		According Tcl interp.**
  **			Tcl_DString	*cmdptr		Buffer fot the Tcl   **
