@@ -5,7 +5,7 @@
  **   Providing a flexible user environment				     **
  ** 									     **
  **   File:		cmdVersion.c					     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Authors:	Jens Hamisch, jens@Strawberry.COM			     **
  ** 									     **
@@ -47,7 +47,7 @@
  ** 									     ** 
  ** ************************************************************************ **/
 
-static char Id[] = "@(#)$Id: cmdVersion.c,v 1.2 2001/06/09 09:48:46 rkowen Exp $";
+static char Id[] = "@(#)$Id: cmdVersion.c,v 1.9 2005/11/29 04:26:30 rkowen Exp $";
 static void *UseId[] = { &UseId, Id };
 
 /** ************************************************************************ **/
@@ -195,7 +195,7 @@ static	char		*scan_versions(		char 		 *buffer,
  ** 									     **
  **   Description:	Callback function for 'version'			     **
  ** 									     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Parameters:	ClientData	 client_data			     **
  **			Tcl_Interp	*interp		According Tcl interp.**
@@ -216,7 +216,7 @@ static	char		*scan_versions(		char 		 *buffer,
 int	cmdModuleVersion(	ClientData	 client_data,
 	      			Tcl_Interp	*interp,
 	      			int		 argc,
-	      			char		*argv[])
+	      			CONST84 char	*argv[])
 {
     char	*version, *module;
     ModModule	*modptr;
@@ -244,7 +244,7 @@ int	cmdModuleVersion(	ClientData	 client_data,
 	    return( TCL_ERROR);		/** -------- EXIT (FAILURE) -------> **/
     }
 
-    if((char *) NULL == (module = CheckModuleVersion( argv[1]))) {
+    if((char *) NULL == (module = CheckModuleVersion( (char *) argv[1]))) {
 	ErrorLogger( ERR_BADMODNAM, LOC, argv[1], NULL);
 	return( TCL_ERROR);		/** -------- EXIT (FAILURE) -------> **/
     }
@@ -295,14 +295,14 @@ int	cmdModuleVersion(	ClientData	 client_data,
     
     for( i=2; i<argc; i++) {
 
-	if( FindName( argv[ i], modptr->name, &tmp)) {
+	if( FindName( (char *) argv[ i], modptr->name, &tmp)) {
 	    if( OK != ErrorLogger( ERR_DUP_SYMVERS, LOC, argv[ i], NULL))
 		break;
 	    else
 		continue;
 	}
 
-	if((ModName *) NULL == (nameptr = AddName( argv[ i], &modptr->name,
+	if((ModName *)NULL == (nameptr = AddName((char *)argv[ i],&modptr->name,
 	    modptr))) {
 	    if( OK != ErrorLogger( ERR_INTERAL, LOC, NULL))
 		break;
@@ -334,7 +334,7 @@ int	cmdModuleVersion(	ClientData	 client_data,
  ** 									     **
  **   Description:	Callback function for 'version'			     **
  ** 									     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Parameters:	char	*name		Name to be expanded	     **
  ** 									     **
@@ -411,7 +411,7 @@ char	*ExpandVersions( char	*name)
  **			ModName pointer and print them as a list into the    **
  **			passed buffer.					     **
  ** 									     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Parameters:	char	*buffer		Buffer for printing in	     **
  **			ModName	*ptr		Name structure pointer	     **
@@ -495,7 +495,7 @@ static	char	*scan_versions( char		 *buffer,
  **   Description:	Reduce the passed module name into a <mod>/<vers>    **
  **			string						     **
  ** 									     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Parameters:	char	*name		name to be checked	     **
  ** 									     **
@@ -593,7 +593,7 @@ static	char	*CheckModuleVersion( char *name)
  ** 									     **
  **   Description:	Callback function for 'alias'			     **
  ** 									     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Parameters:	ClientData	 client_data			     **
  **			Tcl_Interp	*interp		According Tcl interp.**
@@ -614,7 +614,7 @@ static	char	*CheckModuleVersion( char *name)
 int	cmdModuleAlias(	ClientData	 client_data,
 	      		Tcl_Interp	*interp,
 	      		int		 argc,
-	      		char		*argv[])
+	      		CONST84 char	*argv[])
 {
     ModName	*tmp, *ptr;
     char	*version, *module;
@@ -651,11 +651,11 @@ int	cmdModuleAlias(	ClientData	 client_data,
      **  Conditionally split up the passed <module>/<version> pair.
      **/
 
-    trg_alias = FindName( argv[ 2], aliaslist, &tmp);
+    trg_alias = FindName( (char *) argv[ 2], aliaslist, &tmp);
     if( !trg_alias) {
 
-    	if((char *) NULL == (module = CheckModuleVersion( argv[2])))
-	    module = argv[ 2];
+    	if((char *) NULL == (module = CheckModuleVersion( (char *) argv[2])))
+	    module = (char *) argv[ 2];
 
     	if((char *) NULL != (version = strrchr( module, '/')))
 	    *version++ = '\0';
@@ -666,7 +666,7 @@ int	cmdModuleAlias(	ClientData	 client_data,
      **  If it does, we're finished ...
      **/
 
-    if( ptr = FindName( argv[ 1], aliaslist, &tmp)) {
+    if( ptr = FindName( (char *) argv[ 1], aliaslist, &tmp)) {
 
 	if( !ptr->ptr || !ptr->ptr->name ||
 	    !trg_alias && (!ptr->ptr->module || !ptr->ptr->module->module) ) {
@@ -688,7 +688,8 @@ int	cmdModuleAlias(	ClientData	 client_data,
 	 **  We have to allocate a new alias entry
 	 **/
 
-	if((ModName *) NULL == (ptr = AddName( argv[ 1], &aliaslist, NULL))) {
+	if((ModName *) NULL == (ptr = AddName((char *) argv[ 1],
+		&aliaslist,NULL))) {
 	    ErrorLogger( ERR_INTERAL, LOC, NULL);
 	    return( TCL_ERROR);		/** -------- EXIT (FAILURE) -------> **/
 	}
@@ -727,7 +728,7 @@ int	cmdModuleAlias(	ClientData	 client_data,
  ** 									     **
  **   Description:	Resolves a given alias to a module/version string    **
  ** 									     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Parameters:	char	*alias		Name of the alias to be re-  **
  **						solved			     **
@@ -806,7 +807,7 @@ int	AliasLookup(	char	*alias,
  ** 									     **
  **   Description:	Resolves a given alias to a module/version string    **
  ** 									     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Parameters:	char	*alias		Name of the alias to be re-  **
  **						solved			     **
@@ -866,7 +867,6 @@ int	VersionLookup(	char *name, char **module, char **version)
      **  We call it success, if we do not find a registerd name.
      **  In this case <module>/<version> will be returned as passed.
      **/
-
     if((ModModule *) NULL == (mptr = FindModule( *module, &mtmp))) {
 	return( 1);			/** -------- EXIT (SUCCESS) -------> **/
     }
@@ -874,7 +874,6 @@ int	VersionLookup(	char *name, char **module, char **version)
     /**
      **  This is for preventing from endless loops
      **/
-
     histsize = HISTTAB;
     histndx = 0;
 
@@ -889,13 +888,11 @@ int	VersionLookup(	char *name, char **module, char **version)
      **  thing is found, check if the related version record itsself relates
      **  to a name record ...
      **/
-
     while( 1) {
 
 	/**
 	 **  Check the symbolic names ...
 	 **/
-
 	if((ModName *) NULL != (vptr = FindName( *version, mptr->name, &vtmp))){
 	    if( !vptr->version || !vptr->version->name) {
 		if( OK != ErrorLogger( ERR_INTERAL, LOC, NULL)) 
@@ -908,7 +905,6 @@ int	VersionLookup(	char *name, char **module, char **version)
 	    /**
 	     **  Prevent from looping ...
 	     **/
-
 	    for( i=0; i<histndx; i++) {
 		if( history[ i] == vptr) {		/** That's the loop  **/
 		    ErrorLogger( ERR_SYMLOOP, LOC, *version, NULL);
@@ -942,8 +938,7 @@ int	VersionLookup(	char *name, char **module, char **version)
      **  Free the loop preventing list
      **  If version is NULL now, something went wrong in the lookup loop above
      **/
-
-    free( history);
+    null_free((void *) &history);
     return((char *) NULL != *version);
 
 } /** End of 'VersionLookup' **/
@@ -955,7 +950,7 @@ int	VersionLookup(	char *name, char **module, char **version)
  ** 									     **
  **   Description:	Cleanup the version structure			     **
  ** 									     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Parameters:	-						     **
  ** 									     **
@@ -998,7 +993,7 @@ static void	CleanupVersionSub( ModModule *ptr)
 
     CleanupName( ptr->version);
     CleanupName( ptr->name);
-    free( ptr->module);
+    null_free((void *) &(ptr->module));
 
 } /** End of 'CleanupVersionSub' **/
 
@@ -1017,7 +1012,7 @@ static void	CleanupName( ModName *ptr)
      **  Cleanup everything that relates to this record
      **/
 
-    free( ptr->name);
+    null_free((void *) &(ptr->name));
 
 } /** End of 'CleanupName' **/
 
@@ -1028,7 +1023,7 @@ static void	CleanupName( ModName *ptr)
  ** 									     **
  **   Description:	Add a new entry to the modules queue		     **
  ** 									     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Parameters:	char	*name		Name of the new module	     **
  ** 									     **
@@ -1078,7 +1073,7 @@ static	ModModule	*AddModule(	char	*name)
 
     if((char *) NULL == (ptr->module = strdup( name))) {
 	ErrorLogger( ERR_ALLOC, LOC, NULL);
-	free( ptr);
+	null_free((void *) &ptr);
 	return((ModModule *) NULL);
     }
 
@@ -1112,7 +1107,7 @@ static	ModModule	*AddModule(	char	*name)
  ** 									     **
  **   Description:	Find a new entry in the modules queue		     **
  ** 									     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Parameters:	char		*name	Name of be found	     **
  **			ModModule	**prev	Buffer for the 'previous'    **
@@ -1157,7 +1152,7 @@ static	ModModule	*FindModule(	char		 *name,
  ** 									     **
  **   Description:	Add a new entry to the name queue		     **
  ** 									     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Parameters:	char	 *name		Name of the new entry	     **
  **			ModName	**start		Start of the queue	     **
@@ -1202,7 +1197,7 @@ static	ModName	*AddName(	char	 *name,
 
     if((char *) NULL == (ptr->name = strdup( name))) {
 	ErrorLogger( ERR_ALLOC, LOC, NULL);
-	free( ptr);
+	null_free((void *) &ptr);
 	return((ModName *) NULL);
     }
 
@@ -1236,7 +1231,7 @@ static	ModName	*AddName(	char	 *name,
  ** 									     **
  **   Description:	Find a new entry in the modules queue		     **
  ** 									     **
- **   First Edition:	95/12/28					     **
+ **   First Edition:	1995/12/28					     **
  ** 									     **
  **   Parameters:	char	 *name		Name of be found	     **
  **			ModName  *start		Start of the name queue      **
