@@ -5,7 +5,7 @@
  **   Providing a flexible user environment				     **
  ** 									     **
  **   File:		cmdXResource.c					     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Authors:	John Furlan, jlf@behere.com				     **
  **		Leif Hedstrom<hedstrom"@boot.org>			     **
@@ -40,7 +40,7 @@
  ** 									     ** 
  ** ************************************************************************ **/
 
-static char Id[] = "@(#)$Id: cmdXResource.c,v 1.2 2001/06/09 09:48:46 rkowen Exp $";
+static char Id[] = "@(#)$Id: cmdXResource.c,v 1.7 2006/01/17 22:51:34 rkowen Exp $";
 static void *UseId[] = { &UseId, Id };
 
 /** ************************************************************************ **/
@@ -52,7 +52,6 @@ static void *UseId[] = { &UseId, Id };
 #ifdef HAS_X11LIBS
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
-#include <X11/Xmu/SysUtil.h>
 #endif
 
 /** ************************************************************************ **/
@@ -145,9 +144,9 @@ static	ErrType	initBuffers(Tcl_Interp*, register int );
  **   Description:	Adds DEFINES to the define buffer. This code is main-**
  **			ly the same as in the original xrdb.c		     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
- **   Parameters:	char	*title		Name of the ressource	     **
+ **   Parameters:	char	*title		Name of the resource	     **
  **			char	*value		and its value		     **
  ** 									     **
  **   Result:		-						     **
@@ -216,7 +215,7 @@ static	void	addNum(	char	*title,
  **   Description:	Put the client and server specific defines on the    **
  **			define buffer					     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	-						     **
  ** 									     **
@@ -244,7 +243,7 @@ static	void	doDisplayDefines()
      **  faults to the client.
      **/
 
-    XmuGetHostname( client, MAXHOSTNAME);
+    gethostname(client,MAXHOSTNAME);
     strcpy( server, XDisplayName( NULL));
 
     if( colon = strchr( server, ':'))
@@ -273,7 +272,7 @@ static	void	doDisplayDefines()
  ** 									     **
  **   Description:	Put the screen specific defines on the define buffer **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	int	scrno	Screen number			     **
  ** 									     **
@@ -355,7 +354,7 @@ static	void	 doScreenDefines( int	scrno)
  **   Note:		This routine uses the global variable 'line', declar-**
  **			ed in another file!!!				     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	register FILE	*input	The stream to be read from   **
  **			int		 do_cpp	Differs betweem a pipe or a  **
@@ -364,7 +363,7 @@ static	void	 doScreenDefines( int	scrno)
  **   Result:		-						     **
  ** 									     **
  **   Attached Globals:	line		Buffer for a line to be read	     **
- **			buffer		Buffer for the whole ressource file  **
+ **			buffer		Buffer for the whole resource file  **
  **					image				     **
  ** 									     **
  ** ************************************************************************ **
@@ -401,19 +400,19 @@ static	int	readFile(	register FILE	*input,
  ** 									     **
  **   Function:		getEntries					     **
  ** 									     **
- **   Description:	Updates the ressources database (which is a Tcl hash **
- **			table) with the ressources passed in the buffer. The **
- **			buffer contains a X ressource lookalike text image.  **
+ **   Description:	Updates the resources database (which is a Tcl hash **
+ **			table) with the resources passed in the buffer. The **
+ **			buffer contains a X resource lookalike text image.  **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		According Tcl interp.**
  **			Tcl_HashTable	*data	The hash tables holding the  **
- **						ressource data		     **
+ **						resource data		     **
  **			register char	*buf	The buffer containing the    **
- **						ressources to be modified in **
- **						X ressource syntax	     **
- **			int		 remove	Remove or add ressources     **
+ **						resources to be modified in **
+ **						X resource syntax	     **
+ **			int		 remove	Remove or add resources     **
  ** 									     **
  **   Result:		ErrType	NO_ERR		Success			     **
  **				ERR_PARSE	Parse error		     **
@@ -428,7 +427,7 @@ static	ErrType getEntries(	Tcl_Interp	*interp,
 				register char	*buf,
 				int		 remove)
 {
-    static Tcl_RegExp		res_exp = (Tcl_RegExp) NULL;
+    Tcl_RegExp		res_exp = (Tcl_RegExp) NULL;
     register Tcl_HashEntry	*entry;
     char			*end;
     int				 new_res;
@@ -442,14 +441,14 @@ static	ErrType getEntries(	Tcl_Interp	*interp,
      **
      **       <resource>:	<value>
      **
-     **  The ressource will be returned as \1 and the value as \2
+     **  The resource will be returned as \1 and the value as \2
      **  Set the regexp pointer only, if it hasn't already been set. This 
      **  is a constant regexp!
      **/
 
     if( !res_exp)
 	res_exp  = Tcl_RegExpCompile(interp,
-		 "[ \t]*([^ \t]*)[ \t]*:[ \t]*(.*)[ \t]*");
+		 "^[ \t]*([^ \t]*)[ \t]*:[ \t]*(.*)[ \t]*$");
 
     /**
      **  Seek for the lines (buffers) end. Put a terminator there. Take care of
@@ -473,7 +472,7 @@ static	ErrType getEntries(	Tcl_Interp	*interp,
 	    while( *buf++) ;
 
 	/**
-	 **  Otherwise we're seeking for a syntacticl correct X ressource entry
+	 **  Otherwise we're seeking for a syntacticl correct X resource entry
 	 **/
 
 	} else if( !Tcl_RegExpExec(interp, res_exp, buf, buf)) {
@@ -486,33 +485,39 @@ static	ErrType getEntries(	Tcl_Interp	*interp,
 	    /**
 	     **  Valid entry found. Set up buf pointing behind the pattern
 	     **  that has matched and put a terminator at the end of either the
-	     **  ressource name and its value.
+	     **  resource name and its value.
 	     **/
 
 	    char *startp, *endp;
-	    Tcl_RegExpRange(res_exp, 0, &startp, &endp);
+	    Tcl_RegExpRange(res_exp, 0,
+		(CONST84 char **) &startp, (CONST84 char **) &endp);
 	    buf = endp + 1;
-	    Tcl_RegExpRange(res_exp, 1, &startp, &endp);
+	    Tcl_RegExpRange(res_exp, 1,
+		(CONST84 char **) &startp, (CONST84 char **) &endp);
 	    *endp = '\0';
-	    Tcl_RegExpRange(res_exp, 2, &startp, &endp);
+	    Tcl_RegExpRange(res_exp, 2,
+		(CONST84 char **) &startp, (CONST84 char **) &endp);
 	    *endp = '\0';
 
 	    /**
-	     **  Now create (or remove) a hash entry for the parsed ressource
+	     **  Now create (or remove) a hash entry for the parsed resource
 	     **/
 
 	    if( remove) {
-	        Tcl_RegExpRange(res_exp, 1, &startp, &endp);
+	        Tcl_RegExpRange(res_exp, 1,
+			(CONST84 char **) &startp, (CONST84 char **) &endp);
 		if( entry = Tcl_FindHashEntry( data, startp)) {
-		    free( Tcl_GetHashValue( entry));
+		    null_free((void *) &( Tcl_GetHashValue( entry)));
 		    Tcl_DeleteHashEntry( entry);
 		}
 	    } else {
-	        Tcl_RegExpRange(res_exp, 1, &startp, &endp);
+	        Tcl_RegExpRange(res_exp, 1,
+			(CONST84 char **) &startp, (CONST84 char **) &endp);
 		entry = Tcl_CreateHashEntry( data, startp, &new_res);
 		if( !new_res)
-		    free( Tcl_GetHashValue( entry));
-	        Tcl_RegExpRange(res_exp, 2, &startp, &endp);
+		    null_free((void *) &( Tcl_GetHashValue( entry)));
+	        Tcl_RegExpRange(res_exp, 2,
+			(CONST84 char **) &startp, (CONST84 char **) &endp);
 		Tcl_SetHashValue( entry, strdup( startp));
 	    }
 
@@ -535,9 +540,9 @@ static	ErrType getEntries(	Tcl_Interp	*interp,
  **   Description:	Update the X11 resource property, adding new resour- **
  **			ces.						     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
- **   Parameters:	register ResourceDB *rdb	Ressource database   **
+ **   Parameters:	register ResourceDB *rdb	Resource database   **
  ** 									     **
  **   Result:		-						     **
  ** 									     **
@@ -561,10 +566,10 @@ static	void	storeResProp(	register ResourceDB *rdb)
 #endif
 
     /**
-     **	 Write all attached ressources into the buffer. Follow the X 
-     **  ressource syntax:
+     **	 Write all attached resources into the buffer. Follow the X 
+     **  resource syntax:
      **
-     **        <ressource>:	<value>
+     **        <resource>:	<value>
      **/
 
     Tcl_DStringTrunc( buffer, 0);
@@ -616,9 +621,9 @@ static	void	storeResProp(	register ResourceDB *rdb)
  **			the DISPLAY. This routine should only be called if   **
  **			resDB.data is NULL.				     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
- **   Parameters:	register char	**buf	Buffer for the old ressource **
+ **   Parameters:	register char	**buf	Buffer for the old resource **
  **						database		     **
  ** 									     **
  **   Result:		ErrType	ERR_PARAM	resDB.data != NULL	     **
@@ -644,7 +649,7 @@ static	ErrType getOld( register char **buf)
      **/
 
     if( resDB.data)
-	if( OK != ErrorLogger( ERR_PARAM, LOC, "Ressource database", NULL))
+	if( OK != ErrorLogger( ERR_PARAM, LOC, "Resource database", NULL))
 	    return( ERR_PARAM);		/** ------- EXIT (PARAMETER) -----> **/
 
     if( !(resDB.data = (Tcl_HashTable *) malloc( sizeof( Tcl_HashTable))))
@@ -652,7 +657,7 @@ static	ErrType getOld( register char **buf)
 	    return( ERR_ALLOC);		/** ----- EXIT (OUT OF MEMORY) ----> **/
 
     /**
-     **  Initialize the hash table and read in the old ressources
+     **  Initialize the hash table and read in the old resources
      **/
 
     Tcl_InitHashTable( resDB.data, TCL_STRING_KEYS);
@@ -676,12 +681,12 @@ static	ErrType getOld( register char **buf)
  **   Description:	Initilize buffers if not already done, or reinitia-  **
  **			lize some variables if buffers already exists.	     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		According Tcl interp.**
  **   			register int is_file	Differs between a single X   **
- **						ressource to be modified or  **
- **						a ressource file to be merged**
+ **						resource to be modified or  **
+ **						a resource file to be merged**
  ** 									     **
  **   Result:		ErrType	ERR_DISPLAY	Cannot open DISPLAY	     **
  **				ERR_ALLOC	ALLOC failure		     **
@@ -689,7 +694,7 @@ static	ErrType getOld( register char **buf)
  **				NO_ERR		Success			     **
  ** 									     **
  **   Attached Globals:	dpy		Display will be openend		     **
- **			resDB		Ressource database will be filled up **
+ **			resDB		Resource database will be filled up **
  **					with the current setup		     **
  **			defines						     **
  ** 									     **
@@ -714,7 +719,7 @@ static	ErrType	initBuffers(	Tcl_Interp *interp,
 	    return( ERR_DISPLAY);	/** -------- EXIT (FAILURE) -------> **/
 
     /**
-     **  Read in the old setup at first and put it into the ressource database
+     **  Read in the old setup at first and put it into the resource database
      **/
 
     if( !resDB.data) {
@@ -783,7 +788,7 @@ static	ErrType	initBuffers(	Tcl_Interp *interp,
  **			always will be called, even if there was no 	     **
  **			"x-resource" command in the module!		     **
  ** 									     **
- **   First Edition:    91/10/23                                             **
+ **   First Edition:    1991/10/23                                             **
  **                                                                          **
  **   Parameters:                                                            **
  **                                                                          **
@@ -804,7 +809,7 @@ void xresourceFinish(register int no_errors)
 #ifdef HAS_X11LIBS
 
     /**
-     **  If there is data stored in the ressource database, spool it to the
+     **  If there is data stored in the resource database, spool it to the
      **  according X server
      **/
 
@@ -829,13 +834,13 @@ void xresourceFinish(register int no_errors)
  ** 									     **
  **   Function:	 	cmdXResource					     **
  ** 									     **
- **   Description:	Callback function for 'x-ressource'. The function    **
- **			sets up a hash table containing all ressources to be **
+ **   Description:	Callback function for 'x-resource'. The function    **
+ **			sets up a hash table containing all resources to be **
  **			passed to the X server. This hash table will be	     **
  **			flushed whenever the function xresourceFinish is cal-**
  **			led.						     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	ClientData	 client_data			     **
  **			Tcl_Interp	*interp		According Tcl interp.**
@@ -855,7 +860,7 @@ void xresourceFinish(register int no_errors)
 int	cmdXResource(	ClientData	 client_data,
 			Tcl_Interp	*interp,
 			int		 argc,
-			char		*argv[])
+			CONST84 char	*argv[])
 {
     register FILE	*inp;
     int			 is_file, i,
@@ -872,6 +877,12 @@ int	cmdXResource(	ClientData	 client_data,
 
     if( g_flags & (M_WHATIS | M_HELP))
         return( TCL_OK);		/** ------- EXIT PROCEDURE -------> **/
+
+    if( !getenv("DISPLAY")) {
+      /* don't bother trying to set display variables, if there is no display */
+      return(TCL_OK);
+    }
+    
 	
     /**
      **  Parameter check
@@ -888,8 +899,8 @@ int	cmdXResource(	ClientData	 client_data,
     }
 
     /**
-     **  Ok, now let's treat all remaining arguments as X ressources or
-     **  X ressource files. At first let's check if it is a file ...
+     **  Ok, now let's treat all remaining arguments as X resources or
+     **  X resource files. At first let's check if it is a file ...
      **/
 
     while( opt_ind < argc) {
@@ -913,7 +924,7 @@ int	cmdXResource(	ClientData	 client_data,
 		return( TCL_ERROR);	/** -------- EXIT (FAILURE) -------> **/
 
 	    /**
-	     **  This puts all required ressources into a text image buffer ...
+	     **  This puts all required resources into a text image buffer ...
 	     **/
 
 	    if( !is_file) {
