@@ -5,7 +5,7 @@
  **   Providing a flexible user environment				     **
  ** 									     **
  **   File:		error.c						     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Authors:	Jens Hamisch, jens@Strawberry.COM			     **
  ** 									     **
@@ -30,7 +30,7 @@
  ** 									     ** 
  ** ************************************************************************ **/
 
-static char Id[] = "@(#)$Id: error.c,v 1.2 2001/06/09 09:48:46 rkowen Exp $";
+static char Id[] = "@(#)$Id: error.c,v 1.8.6.1.4.1 2006/04/20 22:35:33 rkowen Exp $";
 static void *UseId[] = { &UseId, Id };
 
 /** ************************************************************************ **/
@@ -250,7 +250,7 @@ static	ErrTransTab	TransTab[] = {
     { NO_ERR_VERBOSE,	WGHT_VERBOSE,  NULL },
     { ERR_PARAM,	WGHT_ERROR, "Paramter error concerning '$1'" },
     { ERR_USAGE,	WGHT_ERROR, "Usage is '$*'" },
-    { ERR_ARGSTOLONG,	WGHT_ERROR, "'$1': Arguments to long. Max. is '$2'" },
+    { ERR_ARGSTOLONG,	WGHT_ERROR, "'$1': Arguments too long. Max. is '$2'" },
     { ERR_OPT_AMBIG,	WGHT_ERROR, "Option '$1' is ambiguous" },
     { ERR_OPT_NOARG,	WGHT_ERROR, "Option '$1' allows no argument" },
     { ERR_OPT_REQARG,	WGHT_ERROR, "Option '$1' requires an argument" },
@@ -284,6 +284,7 @@ static	ErrTransTab	TransTab[] = {
     { ERR_UNAME,	WGHT_FATAL, "'uname (2)' failed." },
     { ERR_GETHOSTNAME,	WGHT_FATAL, "'gethostname (2)' failed." },
     { ERR_GETDOMAINNAME,WGHT_FATAL, "'getdomainname (2)' failed." },
+    { ERR_STRING,	WGHT_FATAL, "string error" },
     { ERR_DISPLAY,	WGHT_ERROR, "Cannot open display" },
     { ERR_PARSE,	WGHT_ERROR, "Parse error" },
     { ERR_EXEC,		WGHT_ERROR, "Tcl command execution failed: $1" },
@@ -306,6 +307,19 @@ static	ErrTransTab	TransTab[] = {
     { ERR_DUP_ALIAS,	WGHT_WARN,  "Duplicate alias '$1' found" },
     { ERR_CACHE_INVAL,	WGHT_ERROR, "Invalid cache version '$1' found" },
     { ERR_CACHE_LOAD,	WGHT_WARN,  "Couldn't load the cache properly" },
+    { ERR_BEGINENV,	WGHT_WARN, "Invalid update subcommand - "
+#ifdef BEGINENV
+#  if BEGINENV == 99
+	"No _MODULESBEGINENV_ file"
+#  else
+	"No .modulesbeginenv file"
+#  endif
+#else
+	".modulesbeginenv not supported"
+#endif
+	},
+    { ERR_BEGINENVX,	WGHT_WARN,
+	"Invalid update subcommand - No MODULESBEGINENV - hence not supported"},
     { ERR_INIT_TCL,	WGHT_ERROR, "Cannot initialize TCL" },
     { ERR_INIT_TCLX,	WGHT_WARN,  "Cannot initialize TCLX modules using "
 				    "extended commands might fail" },
@@ -324,6 +338,7 @@ static	ErrTransTab	TransTab[] = {
     { ERR_INVAL,	WGHT_PANIC, "Invalid error type '$1' found" },
     { ERR_INVWGHT,	WGHT_PANIC, "Invalid error weight '$1' found" },
     { ERR_INVFAC,	WGHT_PANIC, "Invalid log facility '$1'" },
+    { ERR_ENVVAR,       WGHT_FATAL, "The environment variables LOADMODULES and _LMFILES_ have inconsistent lengths." }
 };
 
 /** ************************************************************************ **/
@@ -384,7 +399,7 @@ static	void Print_Tracing(	char	 *buffer,
  ** 									     **
  **   Description:	Display a tracing or verbose message		     **
  ** 									     **
- **   First Edition:	95/12/27					     **
+ **   First Edition:	1995/12/27					     **
  ** 									     **
  **   Parameters:	int	result	Result code of th module command     **
  **			int	argc	Number od arguments to the module    **
@@ -545,7 +560,7 @@ void Restore_Error(void) {
  ** 									     **
  **   Description:	Error handling for the modules package		     **
  ** 									     **
- **   First Edition:	95/08/06					     **
+ **   First Edition:	1995/08/06					     **
  ** 									     **
  **   Parameters:	ErrType		 error_type	Type of the error    **
  **			char		*module		Affected module	     **
@@ -584,7 +599,6 @@ int Module_Error(	ErrType		  error_type,
     /**
      **  Argument check
      **/
-
     if( NO_ERR_VERBOSE == error_type && !sw_verbose)
 	return( OK);
 
@@ -607,7 +621,6 @@ int Module_Error(	ErrType		  error_type,
 	/**
 	 **  Conditionally realloc
 	 **/
-
 	while( argc >= listsize) {
 	    listsize += ARGLIST_SIZE;
 	    if( NULL == (argv = (char **) realloc( argv,
@@ -630,10 +643,9 @@ int Module_Error(	ErrType		  error_type,
      **  Locate the error translation table entry according to the 
      **  passed error type
      **/
-
     if( NULL == (TransPtr = ErrorLookup( error_type))) {
 	if( argv)
-	    free( argv);
+	    null_free((void *) &argv);
 	return( ErrorLogger( ERR_INVAL, LOC, (sprintf( buffer, "%d",
 	    error_type), buffer), NULL));
     }
@@ -644,7 +656,7 @@ int Module_Error(	ErrType		  error_type,
 
     if( NULL == (MeasPtr = MeasLookup( TransPtr->error_weight))) {
 	if( argv)
-	    free( argv);
+	    null_free((void *) &argv);
 	argc = 0;
 	return( ErrorLogger( ERR_INVWGHT, LOC, (sprintf( buffer, "%d",
 	    TransPtr->error_weight), buffer), NULL));
@@ -653,7 +665,6 @@ int Module_Error(	ErrType		  error_type,
     /**
      **  Use the return code as defined for the current user level
      **/
-
     switch( sw_userlvl) {
 	case UL_NOVICE:
 	    ret_code = MeasPtr->ret_nov;
@@ -669,12 +680,11 @@ int Module_Error(	ErrType		  error_type,
     /**
      **  Print the error message
      **/
-
     if( TransPtr->error_weight <= WGHT_TRACE || ret_code != OK)
 	if( !FlushError( error_type, module, lineno, TransPtr->error_weight,
 	    MeasPtr->message, TransPtr->messages, argc, argv)) {
 	    if( argv)
-		free( argv);
+		null_free((void *) &argv);
 	    argc = 0;
 	    return( ErrorLogger( ERR_INTERRL, LOC, NULL));
 	}
@@ -682,12 +692,11 @@ int Module_Error(	ErrType		  error_type,
     /**
      **  Return to the caller ... conditionally ...
      **/
-
     if( WARN == ret_code)
 	ret_code = OK;
 
     if( argv)
-	free( argv);
+	null_free((void *) &argv);
     argc = 0;
 
     if( ret_code > ERROR)
@@ -703,7 +712,7 @@ int Module_Error(	ErrType		  error_type,
  ** 									     **
  **   Description:	Look up the passed error type in the translation tab.**
  ** 									     **
- **   First Edition:	95/08/06					     **
+ **   First Edition:	1995/08/06					     **
  ** 									     **
  **   Parameters:	ErrType		 error_type	Type of the error    **
  ** 									     **
@@ -771,7 +780,7 @@ static	ErrTransTab	*ErrorLookup(	ErrType error_type )
  **   Description:	Look up the passed error weight in the measurement   **
  **			table						     **
  ** 									     **
- **   First Edition:	95/08/06					     **
+ **   First Edition:	1995/08/06					     **
  ** 									     **
  **   Parameters:	ErrWeights	weigth	Weight of the error	     **
  ** 									     **
@@ -840,7 +849,7 @@ static	ErrMeasr	*MeasLookup(	ErrWeights weigth )
  **   Description:	Print the error message. Decide which facility to    **
  **			use and schedule the according logger routine	     **
  ** 									     **
- **   First Edition:	95/12/21					     **
+ **   First Edition:	1995/12/21					     **
  ** 									     **
  **   Parameters:	ErrType		  Type		Error type as passed **
  **			char		 *module	Module name	     **
@@ -876,73 +885,54 @@ static	int	FlushError(	ErrType		  Type,
      **  get the error facilities at first. If there isn't any, we may
      **  return on success immediatelly.
      **/
-
     if((char *) NULL == (facilities = GetFacility( Weight)))
-	return( 1);
+	goto success0;
 
     /**
      **  PANIC and FATAL error messages ought to be on stderr at least!
      **/
-
-    if( WGHT_FATAL == Weight || WGHT_PANIC == Weight) {
+    if( WGHT_FATAL == Weight || WGHT_PANIC == Weight)
 	if( !strstr( facilities, _stderr)) {
 
-	    int x = strlen( facilities);
-
-	    if((char *) NULL == (buffer = (char *) malloc( x + 2 +
-		strlen( _stderr) ))) {
-		ErrorLogger( ERR_ALLOC, LOC, NULL);
-		return( 0);
+	    if((char *) NULL == (buffer = stringer(NULL,0,
+		    _stderr,":", facilities, NULL))) {
+		ErrorLogger( ERR_STRING, LOC, NULL);
+		goto unwind0;
 	    }
 
-	    strcpy( buffer, _stderr);
-	    buffer[ 6] = ':';
-	    strcpy( &buffer[7], facilities);
-
 	    facilities = buffer;
 	    fac_alloc = 1;
 	}
-    }
 
     /**
      **  Print the error message into the buffer
      **/
-
-    if((char *) NULL == (errmsg_buffer = (char *) malloc( ERR_BUFSIZE))) {
+    if((char *) NULL == (errmsg_buffer = stringer(NULL, ERR_BUFSIZE, NULL))) {
 	ErrorLogger( ERR_ALLOC, LOC, NULL);
-	if( fac_alloc)
-	    free( facilities);
-	return( 0);
+	goto unwind1;
     }
 
     /**
      **  In case of verbosity, the first argument is the ErrMsgs format string
      **/
-
     if( WGHT_VERBOSE == Weight) {
 	ErrMsgs = *argv++;
 	--argc;
     }
 
-    if( WGHT_TRACE == Weight) {
+    if( WGHT_TRACE == Weight)
 	Print_Tracing( errmsg_buffer, lineno, argc, argv);
-
-    } else if( !PrintError( errmsg_buffer, Type, module, lineno, Weight,
-	WeightMsg, ErrMsgs, argc, argv)) {
-	free( errmsg_buffer);
-	if( fac_alloc)
-	    free( facilities);
-	return( 0);
-    }
+    else if( !PrintError( errmsg_buffer, Type, module, lineno, Weight,
+	WeightMsg, ErrMsgs, argc, argv))
+	    goto unwind2;
 
     /**
      **  Now tokenize the facilities string and schedule the error messge
      **  for every single facility
      **/
-
-    for( fac = strtok( facilities, ":");
+    for( fac = xstrtok( facilities, ":");
 	 fac;
-	 fac = strtok( (char *) NULL, ":") ) {
+	 fac = xstrtok( (char *) NULL, ":") ) {
 
 	/**
 	 **  Check for filenames. Two specials are defined: stderr and stdout
@@ -951,7 +941,6 @@ static	int	FlushError(	ErrType		  Type,
 	 **  syslog facility
 	 **  'null' and 'none' are known as 'no logging'
 	 **/
-
 	if( !strcmp( fac, _null) || !strcmp( fac, _none))
 	    continue;
 
@@ -964,7 +953,6 @@ static	int	FlushError(	ErrType		  Type,
 	/**
 	 **  Syslog
 	 **/
-
 	else if( '.' != *fac && '/' != *fac) {
 #if defined(HAVE_SYSLOG) && defined(WITH_LOGGING)
 	    int syslog_fac, syslog_lvl;
@@ -979,19 +967,16 @@ static	int	FlushError(	ErrType		  Type,
 	     **  Invalid facilities ... take care not to end up in 
 	     **  infinite loops
 	     **/
-
 	    } else if( Type == ERR_INVFAC ||
 	               OK == ErrorLogger( ERR_INVFAC, LOC, fac, NULL)) 
 		continue;
 
 #else
-#ifdef	SYSLOG_DIR
-	    /* sprintf( buffer, "%s/%s", SYSLOG_DIR, fac); */
-	    strcpy( buffer, SYSLOG_DIR);
-	    strcat( buffer, "/");
-	    strcat( buffer, fac);
+#  ifdef	SYSLOG_DIR
+	    /* this is an intential memory leak */
+	    buffer = stringer(NULL,0, SYSLOG_DIR, "/", fac);
 	    fac = buffer;
-#endif
+#  endif
 #endif
 	}
 
@@ -999,41 +984,28 @@ static	int	FlushError(	ErrType		  Type,
 	 **  Custom files ...
 	 **  This may result from the syslog part above
 	 **/
-
 	if( '.' == *fac || '/' == *fac) {
 	    if((FILE *) NULL == (facfp = fopen( fac, "a"))) {
 
-		if( WGHT_PANIC == Weight) {	/** Avoid endless loops!     **/
-		    free( errmsg_buffer);
-		    if( fac_alloc)
-			free( facilities);
-		    return( 0);
-		}
+		if( WGHT_PANIC == Weight)	/** Avoid endless loops!     **/
+		    goto unwind2;
 
 		/**
 		 **  Invalid facilities ... take care not to end up in 
 		 **  infinite loops
 		 **/
-
 		if( Type == ERR_INVFAC ||
-		    OK == ErrorLogger( ERR_INVFAC, LOC, fac, NULL)) {
+		    OK == ErrorLogger( ERR_INVFAC, LOC, fac, NULL))
 		    continue;
-
-		} else {
-		    free( errmsg_buffer);
-		    if( fac_alloc)
-			free( facilities);
-		    return( 0);
-		}
+		else
+		    goto unwind2;
 	    }
 
 	    fprintf( facfp, "%s", errmsg_buffer);
 
 	    if( EOF == fclose( facfp))
-		if( OK != ErrorLogger( ERR_CLOSE, LOC, fac, NULL)) {
-		    free( errmsg_buffer);
-		    return( 0);
-		}
+		if( OK != ErrorLogger( ERR_CLOSE, LOC, fac, NULL))
+		    goto unwind2;
 
 	}
     } /** for **/
@@ -1041,11 +1013,19 @@ static	int	FlushError(	ErrType		  Type,
     /**
      **  Return on success
      **/
-	
-    free( errmsg_buffer);
+    null_free((void *) &errmsg_buffer);
     if( fac_alloc)
-	free( facilities);
-    return( 1);
+	null_free((void *) &facilities);
+success0:
+    return( 1);				/** -------- EXIT (SUCCESS) -------> **/
+
+unwind2:
+    null_free((void *) &errmsg_buffer);
+unwind1:
+    if( fac_alloc)
+	null_free((void *) &facilities);
+unwind0:
+    return( 0);				/** -------- EXIT (FAILURE) -------> **/
 
 } /** End of 'FlushError' **/
 
@@ -1057,7 +1037,7 @@ static	int	FlushError(	ErrType		  Type,
  **   Description:	Get the log facility according to the passed error   **
  **			weight						     **
  ** 									     **
- **   First Edition:	95/12/21					     **
+ **   First Edition:	1995/12/21					     **
  ** 									     **
  **   Parameters:	ErrWeights	  Weight	Error Weight	     **
  ** 									     **
@@ -1075,7 +1055,6 @@ static	char	*GetFacility(	ErrWeights	  Weight)
     /**
      **  Get the facility table entry at first
      **/
-
     if( !(facility = GetFacility_sub( Weight)))
 	return((char *) NULL);
 
@@ -1084,7 +1063,6 @@ static	char	*GetFacility(	ErrWeights	  Weight)
      **      First of all there may be a custom facilitiy defined
      **	     Otherwise the default facility is to be returned now
      **/
-
     return( facility->facility ? facility->facility : facility->def_facility);
 
 } /** End of 'GetFacility' **/
@@ -1098,7 +1076,6 @@ static	ErrFacilities	*GetFacility_sub(	ErrWeights	  Weight)
      **  Binary search for the passed facility in the Facilities table.
      **  This requires the table to be sorted on ascending error weight
      **/
-
     low = Facilities;
     high = Facilities + (sizeof( Facilities) / sizeof( Facilities[0]));
     save = (ErrFacilities *) NULL;
@@ -1122,7 +1099,6 @@ static	ErrFacilities	*GetFacility_sub(	ErrWeights	  Weight)
      **  We have to check, if we've found something or if there's an internal
      **  error (wrong weight)
      **/
-
     if( mid->Weight != Weight) {
 	if( OK == ErrorLogger( ERR_INVWGHT, LOC, (sprintf( buffer, "%d",
 	    Weight), buffer), NULL))
@@ -1141,7 +1117,7 @@ static	ErrFacilities	*GetFacility_sub(	ErrWeights	  Weight)
  **   Description:	Check the passwd string to be a valid combination    **
  **			of       <syslog_facility>.<syslog_level>	     **
  ** 									     **
- **   First Edition:	95/12/21					     **
+ **   First Edition:	1995/12/21					     **
  ** 									     **
  **   Parameters:	char	*string		Input facility string	     **
  **			int	*facility	Buffer for the real facility **
@@ -1161,51 +1137,45 @@ int	CheckFacility(	char *string, int *facility, int *level)
     /**
      **  We do not want to change the strings ... so allocate a buffer here
      **/
-
-    if((char *) NULL == (buf = strdup( string)))
-	if( OK == ErrorLogger( ERR_ALLOC, LOC, NULL))
-	    return( 0);
+    if((char *) NULL == (buf = stringer(NULL,0, string,NULL)))
+	if( OK == ErrorLogger( ERR_STRING, LOC, NULL))
+	    goto unwind0;
 
     /** 
      **  We cannot use strtok here, because there's one initialized in an
-     **  outter loop!
+     **  outer loop!
      **/
-
     for( s=buf; s && *s && *s != '.'; s++);
-    if( !s || !*s) {
-	free( buf);
-	return( 0);			/** Bad formed string. 		     **/
-    }
+    if( !s || !*s)
+	goto unwind1;
     *s = '\0';
 	
     /**
      **  This should be the facility
      **/
-
     if( -1 == (x = scan_facility( buf, facility_names,
-	(sizeof( facility_names) / sizeof( facility_names[0])) ))) {
-	free( buf);
-	return( 0);
-    }
+	    (sizeof( facility_names) / sizeof( facility_names[0])) )))
+	goto unwind1;
     *facility = x;
 
     /**
      **  This should be the level
      **/
-
     if( -1 == (x = scan_facility( ++s, level_names, 
-	(sizeof( level_names) / sizeof( level_names[0])) ))) {
-	free( buf);
-	return( 0);
-    }
+	    (sizeof( level_names) / sizeof( level_names[0])) )))
+	goto unwind1;
     *level = x;
 
     /**
      **  Success
      **/
+    null_free((void *) &buf);
+    return( 1);				/** -------- EXIT (SUCCESS) -------> **/
 
-    free( buf);
-    return( 1);
+unwind1:
+    null_free((void *) &buf);
+unwind0:
+    return( 0);				/** -------- EXIT (FAILURE) -------> **/
 
 } /** End of 'CheckFacility' **/
 
@@ -1217,7 +1187,7 @@ int	CheckFacility(	char *string, int *facility, int *level)
  **   Description:	Scan the passed facility names table for the given   **
  **			string and pass back the assigned token		     **
  ** 									     **
- **   First Edition:	95/12/21					     **
+ **   First Edition:	1995/12/21					     **
  ** 									     **
  **   Parameters:	char		*s	String to be checked	     **
  **			FacilityNames	*table	Table of valid names and     **
@@ -1242,10 +1212,12 @@ static	int	scan_facility( char *s, FacilityNames *table, int size)
 	int x;			/** Have to use this, because strcmp will    **/
 				/** not return -1 and 1 on Solaris 2.x	     **/
 	save = mid;
-	mid = low + ((high -low) / 2);
+	mid = low + ((high - low) / 2);
 	if( save == mid)
 	    low = mid = high;			/** To prevent endless loops **/
 
+	if (mid == high)
+		return -1;
 	x = strcmp( mid->name, s);
 
 	if( x < 0 )
@@ -1269,11 +1241,11 @@ static	int	scan_facility( char *s, FacilityNames *table, int size)
  **   Description:	Scan the passed facility names table for the given   **
  **			string and pass back the assigned token		     **
  ** 									     **
- **   First Edition:	95/12/21					     **
+ **   First Edition:	1995/12/21					     **
  ** 									     **
  **   Parameters:	char	*facility	Name of the facility	     **
  ** 									     **
- **   Result:		char**	NULL		Invalif facility name	     **
+ **   Result:		char**	NULL		Invalid facility name	     **
  **				Otherwise	Pointer to the facilty string**
  **						reference		     **
  ** 									     **
@@ -1291,20 +1263,17 @@ char	**GetFacilityPtr( char *facility)
      **  Try to figure out the error weight at first
      **  Need the given weight in upper case for this
      **/
-
     len = strlen( facility);
 
-    if((char *) NULL == (buf = malloc( len + 1)))
+    if((char *) NULL == (buf = stringer(NULL, 0, facility, NULL)))
 	if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL))
-	    return((char **) NULL);
+	    goto unwind0;
 
-    s = facility; t = buf;
-    while( *t++ = toupper( *s++));
+    for( t = buf; *t; ++t) *t = toupper(*t);
     
     /**
      **  Now look up the measurements table for the uppercase weight
      **/
-
     i = sizeof( Measurements) / sizeof( Measurements[ 0]);
     measptr = Measurements;
 
@@ -1314,10 +1283,10 @@ char	**GetFacilityPtr( char *facility)
 	i--; measptr++;
     }
 
-    free( buf);
+    null_free((void *) &buf);
 
     if( !i) 					/** not found		     **/
-	return((char **) NULL);
+	goto unwind0;
     
     /**
      **  Now get the facility table entry
@@ -1326,14 +1295,16 @@ char	**GetFacilityPtr( char *facility)
     if((ErrFacilities *) NULL == (facptr = GetFacility_sub(
 	measptr->error_weight))) {
 	ErrorLogger( ERR_INVWGHT_WARN, LOC, facility, NULL); 
-	return((char **) NULL);
+	goto unwind0;
     }
 
     /**
      **  Got it ... return the desired pointer
      **/
+    return( &facptr->facility);		/** -------- EXIT (RESULT)  -------> **/
 
-    return( &facptr->facility);
+unwind0:
+    return((char **) NULL);		/** -------- EXIT (FAILURE) -------> **/
 
 } /** End of 'GetFacilityPtr' **/
 
@@ -1344,7 +1315,7 @@ char	**GetFacilityPtr( char *facility)
  ** 									     **
  **   Description:	Print the error message				     **
  ** 									     **
- **   First Edition:	95/08/06					     **
+ **   First Edition:	1995/08/06					     **
  ** 									     **
  **   Parameters:	char             *errbuffer	Buffer to hold the   **
  **							error messge	     **
@@ -1385,7 +1356,6 @@ static	int	PrintError(	char 		 *errbuffer,
      **  Build the error string at first. Note - we cannot alloc memory any
      **  more!
      **/
-
     if( ERR_ALLOC == Type) 
 	error_string = ErrMsgs;
     else
@@ -1411,7 +1381,6 @@ static	int	PrintError(	char 		 *errbuffer,
     /**
      **  Success
      **/
-
     return( 1);
 
 } /** End of 'PrintError' **/
@@ -1423,7 +1392,7 @@ static	int	PrintError(	char 		 *errbuffer,
  ** 									     **
  **   Description:	Print the error message				     **
  ** 									     **
- **   First Edition:	95/08/06					     **
+ **   First Edition:	1995/08/06					     **
  ** 									     **
  **   Parameters:	char		 *ErrMsgs	Error message	     **
  **			int		  argc		Number of arguments  **
@@ -1451,10 +1420,10 @@ static	char	*ErrorString(	char		 *ErrMsgs,
     /**
      **  Allocate memory if neccessary
      **/
-
     if( !error_line)
-	if( NULL == (error_line = (char *) malloc( strsize = ERR_LINELEN))) {
-	    ErrorLogger( ERR_ALLOC, LOC, NULL);
+	if((char *) NULL ==(error_line = stringer(NULL,strsize = ERR_LINELEN,
+		NULL))){
+	    ErrorLogger( ERR_STRING, LOC, NULL);
 	    return( NULL);
 	}
 
@@ -1463,13 +1432,11 @@ static	char	*ErrorString(	char		 *ErrMsgs,
     /**
      **  Scan the error strings to be printed
      **/
-
     while( *ErrMsgs) {
 
 	/**
 	 **  Check for special characters
 	 **/
-
 	switch( *ErrMsgs) {
 	    case '\\':	if( !backslash) {
 				backslash = 1;
@@ -1490,7 +1457,6 @@ static	char	*ErrorString(	char		 *ErrMsgs,
 	/**
 	 **  Add a single character to the error string
 	 **/
-
 	if( ++len >= strsize - 5) {	/** 5 Bytes for safety		     **/
 	    if( NULL == (error_line = (char *) realloc( error_line,
 		strsize += ERR_LINELEN))) {
@@ -1508,7 +1474,6 @@ static	char	*ErrorString(	char		 *ErrMsgs,
     /**
      **  Success. Return a pointer to the newly created string
      **/
-
     *s++ = '\0';
     return( error_line);
 
@@ -1521,7 +1486,7 @@ static	char	*ErrorString(	char		 *ErrMsgs,
  ** 									     **
  **   Description:	Put an argument to the error string		     **
  ** 									     **
- **   First Edition:	95/08/06					     **
+ **   First Edition:	1995/08/06					     **
  ** 									     **
  **   Parameters:	char		**Control	Parameter control    **
  **			char		**Target	Target to print to   **
@@ -1567,7 +1532,6 @@ static	void	add_param(	char		**Control,
     /**
      **  Has something been found ? If not, print a '$'
      **/
-
     if( s == buffer && !last) {
 
 	if( ++(*Length) >= strsize) {
@@ -1586,7 +1550,6 @@ static	void	add_param(	char		**Control,
 	/**
 	 **  Something has been found. Form the parameter index at first
 	 **/
-
 	if( s == buffer)
 	    index = 0;
 	else
@@ -1600,7 +1563,6 @@ static	void	add_param(	char		**Control,
 	/**
 	 **  Spool them all out
 	 **/
-
 	while( index < last) {
 
 	    len = strlen( argv[ index]);
