@@ -5,7 +5,7 @@
  **   Providing a flexible user environment				     **
  ** 									     **
  **   File:		locate_module.c					     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Authors:	John Furlan, jlf@behere.com				     **
  **		Jens Hamisch, jens@Strawberry.COM			     **
@@ -33,7 +33,7 @@
  ** 									     ** 
  ** ************************************************************************ **/
 
-static char Id[] = "@(#)$Id: locate_module.c,v 1.3 2001/11/05 21:50:41 rkowen Exp $";
+static char Id[] = "@(#)$Id: locate_module.c,v 1.14.4.1 2006/04/20 22:35:33 rkowen Exp $";
 static void *UseId[] = { &UseId, Id };
 
 /** ************************************************************************ **/
@@ -99,7 +99,7 @@ static	char	 *GetModuleName( Tcl_Interp*, char*, char*, char*);
  **			filename list. The function is used as compare func- **
  **			tion for qsort.					     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	const void	*fi1	First filename to compare    **
  **			const void	*fi2	Second filename to compare   **
@@ -131,7 +131,7 @@ static int  filename_compare(	const void	*fi1,
  **			environment variable are searched to find a match    **
  **			for the given name.				     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		Attached Tcl interpr.**
  **			char		*modulename	Name of the module to**
@@ -165,11 +165,9 @@ int Locate_ModuleFile(	Tcl_Interp	*interp,
 					/** environment variable MODULEPATH  **/
     char	*mod, *vers;		/** Module and version name for sym- **/
 					/** bolic name lookup		     **/
-
     /**
      **  If it is a full path name, that's the module file to load.
      **/
-
 #if WITH_DEBUGGING_LOCATE
     ErrorLogger( NO_ERR_START, LOC, _proc_Locate_ModuleFile, "modulename = '",
 	modulename, "'", NULL);
@@ -177,23 +175,25 @@ int Locate_ModuleFile(	Tcl_Interp	*interp,
 
     if( !modulename) 
 	if( OK != ErrorLogger( ERR_PARAM, LOC, "modulename", NULL))
-	    return( TCL_ERROR);		/** -------- EXIT (FAILURE) -------> **/
+	    goto unwind0;
 	
     if( modulename[0] == '/' || modulename[0] == '.') {
 
 	p = (char*) strrchr( modulename, '/');
         if(p) {
             *p = '\0';
-	
 	    /**
 	     **  Check, if what has been specified is a valid version of
 	     **  the specified module ...
 	     **/
-
-            if( NULL == (result = GetModuleName(interp, modulename, NULL,
-		(p + 1)))) 
-		return( TCL_ERROR);	/** --- EXIT PROCEDURE (FAILURE) --> **/
-	
+            if((char *) NULL ==
+		(result = GetModuleName(interp, modulename, NULL,(p+1)))) 
+		goto unwind0;
+	    /**
+	     **  Reinstall the 'modulefile' which has been corrupted by
+	     **   tokenization
+	     **/
+	    *p = '/';
 	    /**
 	     **  Reinstall the 'modulefile' which has been corrupted by
 	     **  tokenization
@@ -204,53 +204,47 @@ int Locate_ModuleFile(	Tcl_Interp	*interp,
 	     **  ... Looks good! Conditionally (if there has been no version
 	     **  specified) we have to add the default version
 	     **/
-
             if( !strcmp((p + 1), result)) {
-                strcpy( filename, modulename);
+                if ((char *) NULL == stringer( filename, MOD_BUFSIZE,
+		modulename, NULL))
+		    goto unwind1;
             } else {
-                /* sprintf( filename, "%s/%s", modulename, result); */
-                strcpy( filename, modulename);
-                strcat( filename, "/");
-                strcat( filename, result);
+                if ((char *) NULL == stringer( filename, MOD_BUFSIZE,
+		modulename,"/",result, NULL))
+		    goto unwind1;
             }
- 
         } else {
-
 	    /**
 	     **  Hmm! There's no backslash in 'modulename'. So it MUST begin
 	     **  on '.' and MUST be part of the current directory
 	     **/
-
             if( NULL == (result = GetModuleName( interp, modulename, NULL,
 		modulename)))
-		return( TCL_ERROR);	/** --- EXIT PROCEDURE (FAILURE) --> **/
+		goto unwind0;
 	
             if( !strcmp( modulename, result) ||
 		(strlen( modulename) + 1 + strlen( result) + 1 > MOD_BUFSIZE)) {
-                strncpy( filename, modulename, MOD_BUFSIZE);
+                if ((char *) NULL == stringer( filename, MOD_BUFSIZE,
+		modulename, NULL))
+		    goto unwind1;
             } else {
-                /* sprintf( filename, "%s/%s", modulename, result); */
-                strcpy( filename, modulename);
-                strcat( filename, "/");
-                strcat( filename, result);
+                if ((char *) NULL == stringer( filename, MOD_BUFSIZE,
+		modulename,"/",result, NULL))
+		    goto unwind1;
             }
         }
-
     /**
      **  So it is not a full path name what has been specified. Scan the 
      **  MODULESPATH
      **/
-
     } else {
-
 	/**
 	 **  If I don't find a path in MODULEPATH, there's nothing to search.
 	 **/
-
 	if( !( modulespath = (char *) getenv( "MODULEPATH"))) {
 	    if( OK != ErrorLogger( ERR_MODULE_PATH, LOC, NULL)) {
 		g_current_module = NULL;
-		return( TCL_ERROR);	/** -------- EXIT (FAILURE) -------> **/
+		goto unwind0;
 	    }
 	}
 	/**
@@ -259,96 +253,83 @@ int Locate_ModuleFile(	Tcl_Interp	*interp,
 	{ char *end;
 	if ((char *) NULL != (end = strrchr(modulespath, '\n'))) *end = '\0';
 	}
-
 	/**
 	 **  Expand the module name (in case it is a symbolic one). This must
 	 **  be done once here in order to expand any aliases
 	 **/
-
 	if( VersionLookup( modulename, &mod, &vers)) {
-	    /* sprintf( buf, "%s/%s", mod, vers); */
-	    strcpy( buf, mod);
-	    strcat( buf, "/");
-	    strcat( buf, vers);
+	    if ((char *) NULL == stringer( buf, MOD_BUFSIZE,
+	    mod,"/",vers, NULL))
+		goto unwind0;
 	    modulename = buf;
 	}
-
 	/**
 	 **  Split up the MODULEPATH values into multiple directories
 	 **/
-
 	if( NULL == (pathlist = SplitIntoList(interp, modulespath, &numpaths)))
-	    return( TCL_ERROR);		/** --- EXIT PROCEDURE (FAILURE) --> **/
-
+	    goto unwind0;
 	/**
 	 **  Check each directory to see if it contains the module
 	 **/
-
 	for(i=0; i<numpaths; i++) {
-
 	    if( NULL != (result = GetModuleName( interp, pathlist[i], NULL,
 		modulename))) {
 
 		if( strlen( pathlist[i]) + 2 + strlen( result) > MOD_BUFSIZE) {
-		    strncpy( filename, pathlist[i], MOD_BUFSIZE);
+		    if ((char *) NULL == stringer( filename, MOD_BUFSIZE,
+		    pathlist[i], NULL))
+			goto unwind1;
 		} else {
-		    /* sprintf( filename, "%s/%s", pathlist[i], result); */
-		    strcpy( filename, pathlist[i]);
-		    strcat( filename, "/");
-		    strcat( filename, result);
+		    if ((char *) NULL == stringer( filename, MOD_BUFSIZE,
+		    pathlist[i],"/",result, NULL))
+			goto unwind1;
 		}
-
 		break;
 	    }
-
 	    /**
 	     **  If we havn't found it, we should try to re-expand the module
 	     **  name, because some rc file have been sourced
 	     **/
-
 	    if( VersionLookup( modulename, &mod, &vers)) {
-		/* sprintf( buf, "%s/%s", mod, vers); */
-		strcpy( buf, mod);
-		strcat( buf, "/");
-		strcat( buf, vers);
+                if ((char *) NULL == stringer( buf, MOD_BUFSIZE,
+		mod,"/",vers, NULL))
+		    goto unwind1;
 		modulename = buf;
 	    }
-
 	} /** for **/
-
 	/**
 	 **  Free the memory created from the call to SplitIntoList()
 	 **/
-
 	FreeList( pathlist, numpaths);
-
 	/**
 	 **  If result still NULL, then we really never found it and we should
 	 **  return ERROR and clear the full_path array for cleanliness.
 	 **/
-
 	if( !result) {
 	    filename[0] = '\0';
-	    return( TCL_ERROR);		/** --- EXIT PROCEDURE (FAILURE) --> **/
+	    goto unwind0;
 	}
-
     } /** not a full path name **/	
-
     /**
      **  Free up what has been allocated and pass the result back to
      **  the caller and save the real module file name returned by
      **  GetModuleName
      **/
-
     strncpy( realname, result, MOD_BUFSIZE);
-    realname[ MOD_BUFSIZE-1] = filename[ MOD_BUFSIZE-1] = '\0';
-    free( result);
+    if ((char *) NULL == stringer( realname, MOD_BUFSIZE, result, NULL))
+	goto unwind1;
+    null_free((void *) &result);
 
 #if WITH_DEBUGGING_LOCATE
     ErrorLogger( NO_ERR_END, LOC, _proc_Locate_ModuleFile, NULL);
 #endif
 
     return( TCL_OK);
+
+unwind1:
+    null_free((void *) &result);
+unwind0:
+    return( TCL_ERROR);
 }
 
 /*++++
@@ -361,7 +342,7 @@ int Locate_ModuleFile(	Tcl_Interp	*interp,
  ** 									     **
  **   Notes:		This function is RECURSIVE			     **
  **									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		According Tcl Interp.**
  **			char		*path		Path to start seeking**
@@ -396,184 +377,151 @@ static	char	*GetModuleName(	Tcl_Interp	*interp,
 #if WITH_DEBUGGING_LOCATE_1
     ErrorLogger( NO_ERR_START, LOC, _proc_GetModuleName, NULL);
 #endif
-
     /**
      **  Split the modulename into module and version. Use a private buffer
      **  for this
      **/
-
-    if((char *) NULL == (s = strdup( modulename))) {
+    if((char *) NULL == (s = stringer(NULL, 0,  modulename, NULL))) {
 	ErrorLogger( ERR_ALLOC, LOC, NULL);
-	return( NULL);
+	goto unwind0;
     }
     slen = strlen( s) + 1;
     mod = s;
     if( ver = strchr( mod, '/'))
 	*ver++ = '\0';
-
     /**
      **  Allocate a buffer for full pathname building
      **/
-
-    if( NULL == (fullpath = (char*) malloc( MOD_BUFSIZE))) {
-	if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
-	    free( s);
-	    return( NULL);		/** -------- EXIT (FAILURE) -------> **/
+    if((char *) NULL == (fullpath = stringer(NULL, MOD_BUFSIZE, NULL))) {
+	if( OK != ErrorLogger( ERR_STRING, LOC, NULL)) {
+	    goto unwind1;
 	}
     }
-
     /**
      **  Check whether $path/$prefix/$mod is a directory
      **/
-
     if( prefix) {
-	/* sprintf( fullpath, "%s/%s/%s", path, prefix, mod); */
-	strcpy( fullpath, path);
-	strcat( fullpath, "/");
-	strcat( fullpath, prefix);
-	strcat( fullpath, "/");
-	strcat( fullpath, mod);
+	if((char *) NULL == stringer(fullpath, MOD_BUFSIZE,
+	path,"/",prefix,"/",mod, NULL))
+	    goto unwind1;
     } else {
-	/* sprintf( fullpath, "%s/%s", path, mod); */
-	strcpy( fullpath, path);
-	strcat( fullpath, "/");
-	strcat( fullpath, mod);
+	if((char *) NULL == stringer(fullpath, MOD_BUFSIZE,
+	path,"/",mod, NULL))
+	    goto unwind1;
     }
-
     is_def = !strcmp( mod, _default);
 
     if( is_def || !stat( fullpath, &stats)) {
-
 	/**
 	 **  If it is a directory
 	 **/
-
     	if( !is_def && S_ISDIR( stats.st_mode)) {
-
 	    /**
 	     **  Source the ".modulerc" file if it exists
 	     **  For compatibility source the .version file, too
 	     **/
-
 	    if( prefix) {
-		/* sprintf( modfil_buf, "%s/%s", prefix, mod); */
-		strcpy( modfil_buf, prefix);
-		strcat( modfil_buf, "/");
-		strcat( modfil_buf, mod);
+		if((char *) NULL == stringer(modfil_buf, MOD_BUFSIZE,
+		path,"/",mod, NULL))
+		    goto unwind2;
 	    } else {
-		strcpy( modfil_buf, mod);
+		if((char *) NULL == stringer(modfil_buf, MOD_BUFSIZE,mod, NULL))
+		    goto unwind2;
 	    }
 
-	    /* sprintf( fullpath, "%s/%s", path, modfil_buf); */
-	    strcpy( fullpath, path);
-	    strcat( fullpath, "/");
-	    strcat( fullpath, modfil_buf);
+	    if((char *) NULL == stringer(fullpath, MOD_BUFSIZE,
+	    path,"/",modfil_buf, NULL))
+		goto unwind2;
 	    g_current_module = modfil_buf;
 
 	    if( TCL_ERROR == SourceRC( interp, fullpath, modulerc_file) ||
 		TCL_ERROR == SourceVers( interp, fullpath, modfil_buf)) {
-
-		free( s); free( fullpath);
 		/* flags = save_flags; */
-		return( NULL);		/** -------- EXIT (FAILURE) -------> **/
+		    goto unwind2;
 	    }
-
 	    /**
 	     **  After sourcing the RC files, we have to look up versions again
 	     **/
-
 	    if( VersionLookup( modulename, &mod1, &ver1)) {
 		int len = strlen( mod1) + strlen( ver1) + 2;
-		
 		/**
 		 **  Maybe we have to enlarge s
 		 **/
-
 		if( len > slen) {
-		    if((char *) NULL == (s = realloc( s, len))) {
-			ErrorLogger( ERR_ALLOC, LOC, NULL);
-			free( fullpath);
-			return( NULL);	/** -------- EXIT (FAILURE) -------> **/
+		    null_free((void *) &s);
+		    if((char *) NULL == (s = stringer( NULL, len, NULL))) {
+			ErrorLogger( ERR_STRING, LOC, NULL);
+			goto unwind2;
 		    }
 		    slen = len;
 		}
-
 		/**
 		 **  Print the new module/version in the buffer
 		 **/
-
-		/* sprintf( s, "%s/%s", mod1, ver1); */
-		strcpy( s, mod1);
-		strcat( s, "/");
-		strcat( s, ver1);
-		
+		if((char *) NULL == stringer( s, len, mod1,"/", ver1, NULL)) {
+		    ErrorLogger( ERR_STRING, LOC, NULL);
+		    goto unwind2;
+		}
 		mod = s;
 		if( ver = strchr( s, '/'))
 		    *ver++ = '\0';
 	    }
-
 	    /**
 	     **  recursively delve into subdirectories (until ver == NULL).
 	     **/
-
 	    if( ver) {
 		int len = strlen( mod) + 1;
 
 		if( prefix)
 		    len += strlen( prefix) +1;
-
 		/**
 		 **  Build the new prefix
 		 **/
-
-		if((char *) NULL == (t = (char *) malloc( len))) {
-		    ErrorLogger( ERR_ALLOC, LOC, NULL);
-		    return((char *) NULL);
+		if((char *) NULL == (t = stringer(NULL, len, NULL))) {
+		    ErrorLogger( ERR_STRING, LOC, NULL);
+		    goto unwind2;
 		}
 
 		if( prefix) {
-		    /* sprintf( t, "%s/%s", prefix, mod); */
-		    strcpy( t, prefix);
-		    strcat( t, "/");
-		    strcat( t, mod);
+		    if((char *) NULL == stringer(t, len, prefix,"/",mod, NULL)){
+			ErrorLogger( ERR_STRING, LOC, NULL);
+			goto unwindt;
+		    }
 		} else {
-		    strcpy( t, mod);
+		    if((char *) NULL == stringer(t, len, mod, NULL)){
+			ErrorLogger( ERR_STRING, LOC, NULL);
+			goto unwindt;
+		    }
 		}
-
 		/**
 		 **  This is the recursion
 		 **/
-
 		Result = GetModuleName( interp, path, t, ver);
 
 		/**
 		 **  Free our temporary prefix buffer
 		 **/
-
-		free( t);
+		null_free((void *) &t);
+		if (0) {	/* an error occurred */
+unwindt:
+		    null_free((void *) &t);
+		    goto unwind2;
+		}
 	    } 
-
 	} else {     /** if( $path/$prefix/$mod is a directory) **/
-	
 	    /**
 	     **  Now 'mod' should be either a file or the word 'default'
 	     **  In case of default get the file with the highest verion number
 	     **  in the current directory
 	     **/
-
 	    if( is_def) {
-
 		if( !prefix)
 		    prefix = ".";
-
 		if( NULL == (filelist = SortedDirList( interp, path, prefix,
-		    &numlist))) {
-		    free( fullpath);
-		    free( s);
-		    return( NULL);	/** --- EXIT PROCEDURE (FAILURE) --> **/
-		}
-		prefix = (char *) NULL;
+		&numlist)))
+		    goto unwind1;
 
+		prefix = (char *) NULL;
 		/**
 		 **  Select the first one on the list which is either a
 		 **  modulefile or another directory. We start at the highest
@@ -581,101 +529,89 @@ static	char	*GetModuleName(	Tcl_Interp	*interp,
 		 **  is reverse sorted.
 		 **  If it's a directory, we delve into it.
 		 **/
-
 		for( i=0; i<numlist && Result==NULL; i++) {
-
 		    /**
 		     **  Build the full path name and check if it is a
 		     **  directory. If it is, recursivly try to find there what
 		     **  we're seeking for
 		     **/
-
-		    /* sprintf( fullpath, "%s/%s", path, filelist[ i]); */
-		    strcpy( fullpath, path);
-		    strcat( fullpath, "/");
-		    strcat( fullpath, filelist[ i]);
+		    if ((char *)NULL == stringer(fullpath, MOD_BUFSIZE,
+			path, "/", filelist[i], NULL))
+			    goto unwind2;
 
 		    if( !stat( fullpath, &stats) && S_ISDIR( stats.st_mode)) {
 			Result = GetModuleName( interp, path, prefix,
 			    filelist[ i]);
-
 		    } else {
-
 			/**
 			 **  Otherwise check the file for a magic cookie ...
 			 **/
-
 			if( check_magic( fullpath, MODULES_MAGIC_COOKIE, 
 			    MODULES_MAGIC_COOKIE_LENGTH)) 
 			    Result = filelist[ i];
-
 		    } /** if( !stat) **/
 		} /** for **/
-
 	    } else {  /** default **/
-
 		/**
 		 **  If mod names a file, we have to check wheter it exists and
 		 **  is a valid module file
 		 **/
-
 		if( check_magic( fullpath, MODULES_MAGIC_COOKIE, 
 		    MODULES_MAGIC_COOKIE_LENGTH)) 
 		    Result = mod;
-
 		else {
 		    ErrorLogger( ERR_MAGIC, LOC, fullpath, NULL);
 		    Result = NULL;
 		}
-
 	    } /** if( mod is a filename) **/
-
 	    /**
 	     **  Build the full filename (using prefix and Result) if
 	     **  Result is defined
 	     **/
-
 	    if( Result) {
 		int len = strlen( Result) + 1;
 
 		if( prefix)
 		    len += strlen( prefix) + 1;
 
-		if((char *) NULL == (t = (char *) malloc( len))) {
-		    ErrorLogger( ERR_ALLOC, LOC, NULL);
-		    Result = NULL;
-
-		} else {
-
-		    if( prefix) {
-			/* sprintf( t, "%s/%s", prefix, Result); */
-			strcpy( t, prefix);
-			strcat( t, "/");
-			strcat( t, Result);
-		    } else {
-			strcpy( t, Result);
-		    }
-
-		    Result = t;
+		if((char *) NULL == (t = stringer(NULL, len, NULL))) {
+		   ErrorLogger( ERR_STRING, LOC, NULL);
+		   goto unwind2;
 		}
-	    }
-
+		if( prefix) {
+		    if((char *) NULL == stringer(t,len, prefix,"/",Result,NULL))
+			goto unwindt2;
+		} else {
+		    if((char *) NULL == stringer(t,len, Result,NULL))
+			goto unwindt2;
+		}
+		Result = t;
+		if (0) {	/* an error occurred */
+unwindt2:
+		    null_free((void *) &t);
+		    goto unwind2;
+		}
+	    } 
 	} /** mod is a file **/
     } /** mod exists **/
-
     /**
      **  Free up temporary values and return what we've found
      **/
-
-    free((void*) fullpath);
-    free((void*) s);
+    null_free((void*) &fullpath);
+    null_free((void*) &s);
     FreeList( filelist, numlist);
     
 #if WITH_DEBUGGING_LOCATE_1
     ErrorLogger( NO_ERR_END, LOC, _proc_GetModuleName, NULL);
 #endif
+    return( Result);			/** -------- EXIT (SUCCESS) -------> **/
 
-    return( Result);
+unwind2:
+    null_free((void *) &fullpath);
+unwind1:
+    null_free((void *) &s);
+unwind0:
+    return(NULL);			/** -------- EXIT (FAILURE) -------> **/
 
 } /** End of 'GetModuleName' **/
 
@@ -690,7 +626,7 @@ static	char	*GetModuleName(	Tcl_Interp	*interp,
  **			If the path + the module filename is a directory, the**
  **			directory is read and sorted as the list.	     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		According Tcl Interp.**
  **			char		*path		Path to start seeking**
@@ -729,55 +665,36 @@ char	**SortedDirList(	Tcl_Interp	*interp,
 #if WITH_DEBUGGING_UTIL_1
     ErrorLogger( NO_ERR_START, LOC, _proc_SortedDirList, NULL);
 #endif
-
     /**
      **  Allocate memory for the list to be created. Suggest a list size of
      **  100 Elements. This may be changed later on.
      **/
-
-    if( NULL == (filelist = (char**) calloc( n = 100, sizeof(char*)))) {
+    if( NULL == (filelist = (char**) calloc( n = 100, sizeof(char*))))
 	if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL))
-	    return( NULL);		/** -------- EXIT (FAILURE) -------> **/
-    }
-    
+	    goto unwind0;
     /**
      **  Form the suggested module file name out of the passed path and 
      **  the name of the module. Alloc memory in order to do this.
      **/
-
-    pathlen = strlen( path) + strlen( modulename) + 1;
-    if( NULL == (full_path = (char*) malloc((pathlen+1) * sizeof(char)))) {
-	if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
-	    free( filelist);
-	    return( NULL);		/** --- EXIT PROCEDURE (FAILURE) --> **/
-	}
-    }
-    
-    /* sprintf( full_path, "%s/%s", path, modulename); */
-    strcpy( full_path, path);
-    strcat( full_path, "/");
-    strcat( full_path, modulename);
+    if((char *) NULL == (full_path = stringer(NULL, 0,
+	path,"/",modulename, NULL)))
+	if( OK != ErrorLogger( ERR_STRING, LOC, NULL))
+	    goto unwind0;
+    pathlen = strlen(full_path);
     
 #if WITH_DEBUGGING_UTIL_2
     ErrorLogger( NO_ERR_DEBUG, LOC, "full_path='", full_path, "'", NULL);
 #endif
-
     /**
      **  Check whether this file exists. If it doesn't free up everything
      **  and return on failure
      **/
-
-    if( stat( full_path, &stats)) {
-	FreeList( filelist, n);
-	free((void*) full_path);
-	return( NULL);			/** --- EXIT PROCEDURE (FAILURE) --> **/
-    }
-    
+    if( stat( full_path, &stats))
+	goto unwind2;
     /**
      **  If the suggested module file is a regular one, we've found what we've
      **  seeked for. Put it on the top of the list and return.
      **/
-
     if( S_ISREG( stats.st_mode)) {
 	*listcnt = 1;
 	filelist[0] = strdup( modulename);
@@ -786,170 +703,129 @@ char	**SortedDirList(	Tcl_Interp	*interp,
     ErrorLogger( NO_ERR_DEBUG, LOC, "Module '", modulename, "' found", NULL);
 #endif
 
-	free((void*) full_path);
+	null_free((void*) &full_path);
 	return( filelist);		/** --- EXIT PROCEDURE (SUCCESS) --> **/
     }
-
     /**
      **  What we've found is a directory
      **/
-
     if( S_ISDIR( stats.st_mode)) {
-
-        char	*buf;		/** Buffer for the whole filename for each   **/
+        char	*tbuf;		/** Buffer for the whole filename for each   **/
 				/** content of the directory		     **/
-        char	*mpath;		/** Pointer into *buf where to write the dir **/
+        char	*mpath;		/** Pointer into *tbuf where to write the dir**/
 				/** entry				     **/
 
 	/**
 	 **  Open the directory for reading
 	 **/
-
 	if( NULL == (subdirp = opendir( full_path))) {
 #if 0
 	/* if you can't open the directory ... is that really an error? */
 	    if( OK != ErrorLogger( ERR_OPENDIR, LOC, full_path, NULL))
 #endif
-	    {
-		FreeList( filelist, n);
-		free( full_path);
-		return( NULL);		/** --- EXIT PROCEDURE (FAILURE) --> **/
-	    }
+	    goto unwind2;
 	}
-
 	/**
 	 **  Allocate a buffer for constructing complete file names
-	 **  and initialize it with the directoy part we do already know.
+	 **  and initialize it with the directory part we do already know.
 	 **/
+        if( NULL == (tbuf = stringer(NULL, MOD_BUFSIZE, full_path,"/", NULL)))
+	    if( OK != ErrorLogger( ERR_STRING, LOC, NULL))
+		goto unwind3;
 
-        if( NULL == (buf = (char*) malloc( LINELENGTH * sizeof( char)))) {
-	    if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
-		free( full_path);
-		closedir( subdirp);
-		return( NULL);		/** --- EXIT PROCEDURE (FAILURE) --> **/
-	    }
-        }
-        strcpy( buf, full_path);
-        buf[ pathlen] = '/';
-        mpath = (buf + pathlen + 1);
-
+        mpath = (tbuf + pathlen + 1);
 	/**
-	 **  Now scan all entries of the just openend directory
+	 **  Now scan all entries of the just opened directory
 	 **/
-
 #if WITH_DEBUGGING_UTIL_2
 	ErrorLogger( NO_ERR_DEBUG, LOC, "Reading directory '", full_path, "'", NULL);
 #endif
-
 	for( file = readdir( subdirp), i = 0, j = 0;
 	     file != NULL;
 	     file = readdir( subdirp), i++) {
-
 	    /**
 	     **  Oops! This one exceeds our array. Enlarge it.
 	     **/
-
-	    if( j == n) {
+	    if( j == n)
 		if( NULL == (filelist =
-		    (char**) realloc((char*) filelist, n *= 2))) {
-		    if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
-			free( full_path);
-			free( buf);
-			if( -1 == closedir( subdirp))
-			    ErrorLogger( ERR_CLOSEDIR, LOC, full_path, NULL);
-			return( NULL);	/** --- EXIT PROCEDURE (FAILURE) --> **/
-		    }
-		}
-	    }
-
+		    (char**) realloc((char*) filelist, n *= 2)))
+		    if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL))
+			goto unwindt;
 	    /**
 	     **  Now, if we got a real entry which is not '.*' or '..' and
 	     **  finally is not a temporary file (which are defined to end
 	     **  on '~' ...
 	     **/
-
 	    if( file->d_name                && 
                 *file->d_name != '.'        && 
                 strcmp( file->d_name, "..")  &&
                 file->d_name[ NLENGTH( file) - 1] != '~') {
-
 		/**
 		 **  ... build the full pathname and check for the magic
 		 **  cookie or for another directory level in order to
 		 **  validate this as a modulefile entry we're seeking for.
 		 **/
-
                 strcpy( mpath, file->d_name);
-                if( check_magic( buf, MODULES_MAGIC_COOKIE,
+                if( check_magic( tbuf, MODULES_MAGIC_COOKIE,
 			MODULES_MAGIC_COOKIE_LENGTH) ||
-		    !stat(buf, &stats) ) {
-
+		    !stat(tbuf, &stats) ) {
 		    /**
 		     **  Yep! Found! Put it on the list
 		     **/
-
-                    if( NULL == (filelist[j] = (char*) malloc(
-			strlen( modulename) + NLENGTH(file) + 2)))
-
-			if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
-			    free( full_path);
-			    free( buf);
-			    FreeList( filelist, j);
-			    return( NULL); 	/** ---- EXIT (FAILURE) ----> **/
-			}
-
-                    /* sprintf(filelist[j++], "%s/%s", modulename, file->d_name); */
-                    strcpy(filelist[j], modulename);
-                    strcat(filelist[j], "/");
-                    strcat(filelist[j++], file->d_name);
-
+                    if((char *) NULL == (filelist[j] = stringer(NULL,0,
+			modulename,"/",file->d_name, NULL)))
+			if( OK != ErrorLogger( ERR_STRING, LOC, NULL))
+			    goto unwindt;
+		    j++;
                 } /** if( mag. cookie or directory) **/
 	    } /** if( not a dotfile) **/
 	} /** for **/
-
 	/**
 	 **  Put a terminator at the lists end and then sort the list
 	 **/
-
         filelist[ j] = NULL;
 	qsort( (void*) filelist, (size_t) j, sizeof(char*), filename_compare);
-	
 	/**
 	 **  Free up temporary values ...
 	 **/
-
-	free((void*) full_path);
-	free((void*) buf);
-
 	if( -1 == closedir( subdirp))
 	    if( OK != ErrorLogger( ERR_CLOSEDIR, LOC, full_path, NULL)) {
-		FreeList( filelist, j);
-		return( NULL);		/** --- EXIT PROCEDURE (FAILURE) --> **/
+		goto unwind2;
 	    }
-
+	null_free((void*) &full_path);
+	null_free((void*) &tbuf);
 	/**
 	 **  Set up return values and pass the created list to the caller
 	 **/
-
 	*listcnt = j;
 	return( filelist);		/** --- EXIT PROCEDURE (SUCCESS) --> **/
+	
+	if(0) {	
+unwindt:
+	    null_free((void*) &tbuf);
+	    goto unwind3;
+	}
     }
-
     /**
      **  If it is nor a regular file, neither a directory, we don't support
      **  it at all ...
      **/
-
 /** ??? What about links ??? **/
 
+unwind3:
+    if( -1 == closedir( subdirp))
+	ErrorLogger( ERR_CLOSEDIR, LOC, full_path, NULL);
+unwind2:
+    null_free((void*) &full_path);
+unwind1:
     FreeList( filelist, n);
-    free((void*) full_path);
 
+unwind0:
 #if WITH_DEBUGGING_UTIL_1
     ErrorLogger( NO_ERR_END, LOC, _proc_SortedDirList, NULL);
 #endif
 
-    return( NULL);
+    return( NULL);			/** --- EXIT PROCEDURE (FAILURE) --> **/
 
 } /** End of 'SortedDirList' **/
 
@@ -961,7 +837,7 @@ char	**SortedDirList(	Tcl_Interp	*interp,
  **   Description: 	Splits a path-type environment variable into an array**
  **			of char* list.					     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		According Tcl Interp.**
  **			char		*pathenv	Path to split 	     **
@@ -996,94 +872,69 @@ char	**SplitIntoList(	Tcl_Interp	*interp,
 #if WITH_DEBUGGING_UTIL_1
     ErrorLogger( NO_ERR_START, LOC, _proc_SplitIntoList, NULL);
 #endif
-
     /** 
      **  Paramter check
      **/
-
-    if( !pathenv) {
+    if( !pathenv)
 	if( OK != ErrorLogger( ERR_PARAM, LOC, "pathenv", NULL))
-	    return( NULL);		/** -------- EXIT PROCEDURE -------> **/
-    }
-
+	    goto unwind0;
     /**
      **  Allocate space to copy in the value of the path value to
      **  split. Thus this procedure doesn't change its input parameters.
      **/
-
-    if( NULL == (givenpath = (char*) malloc( strlen( pathenv) + 1))) {
-	if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL))
-	    return( NULL);		/** -------- EXIT PROCEDURE -------> **/
-    }
-
+    if( (char *) NULL == (givenpath = stringer(NULL,0, pathenv,NULL)))
+	if( OK != ErrorLogger( ERR_STRING, LOC, NULL))
+	    goto unwind0;
     /**
      **  Allocate the list which is an array of char*.  n is used to
      **  manage the array's growth if there are more than 100 paths in
      **  the list.
      **  Copy the passed path into the new buffer.
      **/
-
-    if( NULL == (pathlist = (char**) calloc( n = 100, sizeof( char*)))) {
-	if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
-	    free( givenpath);
-	    return( NULL);		/** -------- EXIT PROCEDURE -------> **/
-	}
-    }
-
-    strcpy( givenpath, pathenv);
-
+    if((char **) NULL == (pathlist = (char**) calloc( n = 100, sizeof( char*))))
+	if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL))
+	    goto unwind1;
     /**
      **  Split the given path environment variable into its components.
      **/
-
-    for( i=0, dirname = strtok( givenpath, ": ");
+    for( i=0, dirname = xstrtok( givenpath, ": ");
          dirname;
-	 dirname = strtok( NULL, ": ")) {
-
+	 dirname = xstrtok( NULL, ": ")) {
 	/**
 	 **  Oops! The number of tokens exceed my array - reallocate it
 	 **  and double its size!
 	 **/
-
-	if( i == n) {
-	    if( NULL == (pathlist = (char**) realloc((char*) pathlist,
-		n *= 2))) {
-		if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
-		    free( givenpath);
-		    return( NULL);	/** -------- EXIT PROCEDURE -------> **/
-		}
-	    }
-	}
-
+	if( i == n )
+	    if((char **) NULL == (pathlist = (char**) realloc((char*) pathlist,
+		n *= 2)))
+		if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL))
+		    goto unwind1;
 	/**
 	 **  Put the token into the array. Therefor a new area is allocated for
 	 **  the token using 'xdup' - which expands 1 level of env.vars.
 	 **/
-
 	if( NULL == (pathlist[ i++] = xdup( dirname)))
 	    if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
 		FreeList( pathlist, --i);
-		free( givenpath);
-		return( NULL);		/** -------- EXIT PROCEDURE -------> **/
+		goto unwind1;
 	    }
-
     } /** for **/
-    
     /**
      **  Free up the temporary working array
      **/
-
     if( givenpath)
-	free((void*) givenpath);
-
+	null_free((void*) &givenpath);
     /**
      **  Set up the return value (Number of elements allocated) and pass
      **  the arrays base pointer to the caller
      **/
-
     *numpaths = i;
     return( pathlist);
 
+unwind1:
+    null_free((void *) &givenpath);
+unwind0:
+    return( NULL);			/** -------- EXIT FAILURE -------> **/
 } /** End of 'SplitIntoList' **/
 
 #ifndef FreeList
@@ -1093,13 +944,13 @@ char	**SplitIntoList(	Tcl_Interp	*interp,
  ** 									     **
  **   Function:		FreeList					     **
  ** 									     **
- **   Description:	Frees an char* array type list.			     **
+ **   Description:	Frees a char* array type list.			     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	char	**list		Pointer to the list	     **
- **			int	  numelem	Number of elements in the l. **
- ** 									     **
+ **			int	  numelem	Number of elements in the    **
+ ** 						list			     **
  **   Result:		-						     **
  ** 									     **
  **   Attached Globals:	-						     **
@@ -1115,27 +966,21 @@ void FreeList(	char	**list,
 #if WITH_DEBUGGING_UTIL_2
     ErrorLogger( NO_ERR_START, LOC, _proc_FreeList, NULL);
 #endif
-
     /**
      **  Nothing to do ?
      **/
-
     if( !list)
 	return;
- 
     /**
      **  Free all elements of the list
      **/
-
     for( j = 0; j < numelem; j++)
 	if( list[j] != NULL)
-	    free((void*) list[j]);
- 
+	    null_free((void *) (list + j));
     /**
      **  Free the entire list
      **/
-
-    free((void*) list);
+    null_free((void *) &list);
 
 } /** End of 'FreeList' **/
 
@@ -1148,7 +993,7 @@ void FreeList(	char	**list,
  ** 									     **
  **   Description:	Source the passed global RC file		     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		Tcl interpreter	     **
  **			char		*path		Path to be used      **
@@ -1168,7 +1013,7 @@ int SourceRC( Tcl_Interp *interp, char *path, char *name)
 {
     struct stat	  stats;		/** Buffer for the stat() systemcall **/
     int 	  save_flags, i;
-    char	  buffer[ MOD_BUFSIZE];
+    char	 *buffer;
     int		  Result = TCL_OK;
     static char	**srclist = (char **) NULL;
     static int	  listsize = 0, listndx = 0;
@@ -1178,95 +1023,79 @@ int SourceRC( Tcl_Interp *interp, char *path, char *name)
      **  we do not have to source anything
      **  Only a valid TCL interpreter should be there
      **/
-
     if( !path || !name)
 	return( TCL_OK);
 
     if( !interp)
 	return( TCL_ERROR);
-
     /**
      **  Build the full name of the RC file
      **  Avoid duplicate sourcing
      **/
-
-    /* sprintf( buffer, "%s/%s", path, name); */
-    strcpy( buffer, path);
-    strcat( buffer, "/");
-    strcat( buffer, name);
-
+    if ((char *) NULL == (buffer = stringer(NULL, 0, path,"/",name, NULL)))
+	if( OK != ErrorLogger( ERR_STRING, LOC, NULL))
+	    goto unwind0;
     for( i=0; i<listndx; i++)
 	if( !strcmp( srclist[ i], buffer))
-	    return( TCL_OK);
-
+	    goto unwind1;
     /**
      **  Check whether the RC file exists and has the magic cookie inside
      **/
-
     if( !stat( buffer, &stats)) {
 	if( check_magic( buffer, MODULES_MAGIC_COOKIE, 
 	    MODULES_MAGIC_COOKIE_LENGTH)) {
-
 	    /**
 	     **  Set the flags to 'load only'. This prevents from accidently
 	     **  printing something
 	     **/
-
 	    save_flags = g_flags;
 	    g_flags = M_LOAD;
-
 	    /**
 	     **  Source now
 	     **/
-
 	    if( TCL_ERROR == Execute_TclFile( interp, buffer)) 
 		if( OK != ErrorLogger( ERR_SOURCE, LOC, buffer, NULL)) 
 		    Result = TCL_ERROR;
-
 	    g_flags = save_flags;
-
 	    /**
 	     **  Save the currently sourced file in the list
 	     **  Check whether the list is big enough to fit in a new entry
 	     **/
-
 	    if( !listsize) {
-
 		listsize = SRCFRAG;
 		if((char **) NULL == (srclist = (char **) malloc( listsize *
 		    sizeof( char **)))) {
 		    ErrorLogger( ERR_ALLOC, LOC, NULL);
-		    return( TCL_ERROR);
+		    goto unwind1;
 		}
-
 	    } else if( listndx + 1 >= listsize) {
-
 		listsize += SRCFRAG;
 		if((char **) NULL == (srclist = (char **) realloc( srclist, 
 		    listsize * sizeof( char **)))) {
 		    ErrorLogger( ERR_ALLOC, LOC, NULL);
-		    return( TCL_ERROR);
+		    goto unwind1;
 		}
 	    }
-
 	    /**
 	     **  Put the current RC files name on the list
 	     **/
+	    srclist[ listndx++] = buffer;
 
-	    if((char *) NULL == (srclist[ listndx++] = strdup( buffer)))
-		ErrorLogger( ERR_ALLOC, LOC, NULL);
-
-	} else
+	} else {
 	    ErrorLogger( ERR_MAGIC, LOC, buffer, NULL);
-
+	    null_free((void *) &buffer);
+	}
     } /** if( !stat) **/
-
     /**
      **  Return our result
      **/
-
     return( Result);
 
+unwind1:
+    null_free((void *) &buffer);
+unwind0:
+    return( TCL_ERROR);
+
 } /** End of 'SourceRC' **/
 
 /*++++
@@ -1276,7 +1105,7 @@ int SourceRC( Tcl_Interp *interp, char *path, char *name)
  ** 									     **
  **   Description:	Source the '.version' file			     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		Tcl interpreter	     **
  **			char		*path		Path to be used      **
@@ -1296,7 +1125,7 @@ int SourceVers( Tcl_Interp *interp, char *path, char *name)
 {
     struct stat	  stats;		/** Buffer for the stat() systemcall **/
     int save_flags;
-    char buffer[ MOD_BUFSIZE];
+    char *buffer;
     int Result = TCL_OK;
     char *version;
     char *new_argv[3];
@@ -1307,31 +1136,24 @@ int SourceVers( Tcl_Interp *interp, char *path, char *name)
      **  we do not have to source anything
      **  Only a valid TCL interpreter should be there
      **/
-
     if( !path || !name)
 	return( TCL_OK);
-
     if( !interp)
 	return( TCL_ERROR);
-
     /**
      **  No default version defined so far?
      **/
-
     if( VersionLookup( name, &mod, &ver) &&
 	strcmp( ver, _default))
 	return( TCL_OK);
-
     /**
      **  Build the full name of the RC file and check whether it exists and
      **  has the magic cookie inside
      **/
-
-    /* sprintf( buffer, "%s/%s", path, version_file); */
-    strcpy( buffer, path);
-    strcat( buffer, "/");
-    strcat( buffer, version_file);
-
+    if ((char *) NULL == (buffer = stringer(NULL, 0, path,"/",version_file,
+	NULL)))
+	if( OK != ErrorLogger( ERR_STRING, LOC, NULL))
+	    return( TCL_ERROR);
     if( !stat( buffer, &stats)) {
 	if(
 #if VERSION_MAGIC != 0
@@ -1341,36 +1163,31 @@ int SourceVers( Tcl_Interp *interp, char *path, char *name)
 	1
 #endif
 	) {
-
 	    save_flags = g_flags;
 	    g_flags = M_LOAD;
 
 	    if( TCL_ERROR != (Result = Execute_TclFile( interp, buffer)) && 
-		(version = Tcl_GetVar( interp, "ModulesVersion", 0))) {
-
+		(version = (char *) Tcl_GetVar(interp, "ModulesVersion", 0))) {
 		/**
 		 **  The version has been specified in the
 		 **  '.version' file. Set up the result code
 		 **/
-
-		/* sprintf( buffer, "%s/%s", name, version); */
-		strcpy( buffer, name);
-		strcat( buffer, "/");
-		strcat( buffer, version);
+		null_free((void *) &buffer);
+		if ((char *) NULL == (buffer = stringer(NULL, 0,
+		name,"/",version, NULL)))
+		    if( OK != ErrorLogger( ERR_STRING, LOC, NULL))
+			return( TCL_ERROR);
 
 		new_argv[0] = "module-version";
 		new_argv[1] = buffer;
 		new_argv[2] = _default;
-
 		/**
 		 **  Define the default version
 		 **/
-
 		if( TCL_OK != cmdModuleVersion( (ClientData) 0,
-		    (Tcl_Interp *) NULL, 3, new_argv)) {
+		    (Tcl_Interp *) NULL, 3, (CONST84 char **) new_argv)) {
 			Result = TCL_ERROR;
 		}
-
 	    } /** if( Execute...) **/
 
 	    g_flags = save_flags;
@@ -1379,7 +1196,10 @@ int SourceVers( Tcl_Interp *interp, char *path, char *name)
 	    ErrorLogger( ERR_MAGIC, LOC, buffer, NULL);
 
     } /** if( !stat) **/
-
+    /**
+     ** free buffer memory
+     **/
+    null_free((void *) &buffer);
     /**
      **  Result determines if this was successfull
      **/
